@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2014 CERN
- * Copyright (C) 2016 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2020 KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -24,9 +24,9 @@
 
 #include <geometry/shape_line_chain.h>
 #include <geometry/shape_circle.h>
-#include "layers_id_colors_and_visibility.h"
+#include <math/box2.h>
 
-#include "../class_track.h"
+#include "viatype.h"
 
 #include "pns_item.h"
 
@@ -34,19 +34,32 @@ namespace PNS {
 
 class NODE;
 
+// uniquely identifies a VIA within a NODE without using pointers. Used to
+// simplify (or complexifiy, depending on the point of view) the pointer management
+// in PNS::NODE. Sooner or later I'll have to fix it for good using smart pointers - twl
+struct VIA_HANDLE
+{
+    bool        valid = false;
+    VECTOR2I    pos;
+    LAYER_RANGE layers;
+    int         net;
+};
+
 class VIA : public ITEM
 {
 public:
     VIA() :
         ITEM( VIA_T )
     {
-        m_diameter = 2;     // Dummy value
-        m_drill = 0;
-        m_viaType = VIA_THROUGH;
+        m_diameter = 2; // Dummy value
+        m_drill    = 0;
+        m_viaType  = VIATYPE::THROUGH;
+        m_isFree   = false;
+        m_isVirtual = false;
     }
 
-    VIA( const VECTOR2I& aPos, const LAYER_RANGE& aLayers,
-             int aDiameter, int aDrill, int aNet = -1, VIATYPE_T aViaType = VIA_THROUGH ) :
+    VIA( const VECTOR2I& aPos, const LAYER_RANGE& aLayers, int aDiameter, int aDrill,
+         int aNet = -1, VIATYPE aViaType = VIATYPE::THROUGH ) :
         ITEM( VIA_T )
     {
         SetNet( aNet );
@@ -55,29 +68,27 @@ public:
         m_diameter = aDiameter;
         m_drill = aDrill;
         m_shape = SHAPE_CIRCLE( aPos, aDiameter / 2 );
+        m_hole = SHAPE_CIRCLE( m_pos, aDrill / 2 );
         m_viaType = aViaType;
-
-        //If we're a through-board via, use all layers regardless of the set passed
-        if( aViaType == VIA_THROUGH )
-        {
-            LAYER_RANGE allLayers( 0, MAX_CU_LAYERS - 1 );
-            SetLayers( allLayers );
-        }
+        m_isFree = false;
+        m_isVirtual = false;
     }
 
-
     VIA( const VIA& aB ) :
-        ITEM( VIA_T )
+        ITEM( aB )
     {
         SetNet( aB.Net() );
         SetLayers( aB.Layers() );
         m_pos = aB.m_pos;
         m_diameter = aB.m_diameter;
         m_shape = SHAPE_CIRCLE( m_pos, m_diameter / 2 );
+        m_hole = SHAPE_CIRCLE( m_pos, aB.m_drill / 2 );
         m_marker = aB.m_marker;
         m_rank = aB.m_rank;
         m_drill = aB.m_drill;
         m_viaType = aB.m_viaType;
+        m_isFree = aB.m_isFree;
+        m_isVirtual = aB.m_isVirtual;
     }
 
     static inline bool ClassOf( const ITEM* aItem )
@@ -85,32 +96,19 @@ public:
         return aItem && VIA_T == aItem->Kind();
     }
 
-
-    const VECTOR2I& Pos() const
-    {
-        return m_pos;
-    }
+    const VECTOR2I& Pos() const { return m_pos; }
 
     void SetPos( const VECTOR2I& aPos )
     {
         m_pos = aPos;
         m_shape.SetCenter( aPos );
+        m_hole.SetCenter( aPos );
     }
 
-    VIATYPE_T ViaType() const
-    {
-        return m_viaType;
-    }
+    VIATYPE ViaType() const { return m_viaType; }
+    void SetViaType( VIATYPE aViaType ) { m_viaType = aViaType; }
 
-    void SetViaType( VIATYPE_T aViaType )
-    {
-        m_viaType = aViaType;
-    }
-
-    int Diameter() const
-    {
-        return m_diameter;
-    }
+    int Diameter() const { return m_diameter; }
 
     void SetDiameter( int aDiameter )
     {
@@ -118,30 +116,29 @@ public:
         m_shape.SetRadius( m_diameter / 2 );
     }
 
-    int Drill() const
-    {
-        return m_drill;
-    }
+    int Drill() const { return m_drill; }
 
     void SetDrill( int aDrill )
     {
         m_drill = aDrill;
+        m_hole.SetRadius( m_drill / 2 );
     }
 
-    bool PushoutForce( NODE* aNode,
-            const VECTOR2I& aDirection,
-            VECTOR2I& aForce,
-            bool aSolidsOnly = true,
-            int aMaxIterations = 10 );
+    bool IsFree() const { return m_isFree; }
+    void SetIsFree( bool aIsFree ) { m_isFree = aIsFree; }
 
-    const SHAPE* Shape() const override
-    {
-        return &m_shape;
-    }
+    bool PushoutForce( NODE* aNode, const VECTOR2I& aDirection, VECTOR2I& aForce,
+                       bool aSolidsOnly = true, int aMaxIterations = 10 );
+
+    const SHAPE* Shape() const override { return &m_shape; }
+
+    const SHAPE_CIRCLE* Hole() const override { return &m_hole; }
+    void SetHole( const SHAPE_CIRCLE& aHole ) { m_hole = aHole; }
 
     VIA* Clone() const override;
 
-    const SHAPE_LINE_CHAIN Hull( int aClearance = 0, int aWalkaroundThickness = 0 ) const override;
+    const SHAPE_LINE_CHAIN Hull( int aClearance = 0, int aWalkaroundThickness = 0,
+                                 int aLayer = -1 ) const override;
 
     virtual VECTOR2I Anchor( int n ) const override
     {
@@ -155,12 +152,29 @@ public:
 
     OPT_BOX2I ChangedArea( const VIA* aOther ) const;
 
+    const VIA_HANDLE MakeHandle() const;
+
 private:
-    int m_diameter;
-    int m_drill;
-    VECTOR2I m_pos;
+    int          m_diameter;
+    int          m_drill;
+    VECTOR2I     m_pos;
     SHAPE_CIRCLE m_shape;
-    VIATYPE_T m_viaType;
+    SHAPE_CIRCLE m_hole;
+    VIATYPE      m_viaType;
+    bool         m_isFree;
+
+};
+
+
+class VVIA : public VIA
+{
+public:
+    VVIA( const VECTOR2I& aPos, int aLayer, int aDiameter, int aNet ) :
+        VIA( aPos, LAYER_RANGE( aLayer, aLayer ), aDiameter, aDiameter / 2, aNet )
+    {
+        m_isVirtual = true;
+        SetHole( SHAPE_CIRCLE( Pos(), 1 ) );
+    }
 };
 
 }
