@@ -127,15 +127,18 @@ Block::Block(const UUID &uu, const json &j, IPool &pool, class IBlockProvider &p
 void Block::update_diffpairs()
 {
     for (auto &it : nets) {
-        if (!it.second.diffpair_master)
+        if (!it.second.diffpair_primary)
             it.second.diffpair = nullptr;
     }
     for (auto &it : nets) {
-        if (it.second.diffpair_master) {
-            if (nets.count(it.second.diffpair.uuid))
+        if (it.second.diffpair_primary) {
+            if (nets.count(it.second.diffpair.uuid)) {
                 it.second.diffpair->diffpair = &it.second;
-            else
+            }
+            else {
                 it.second.diffpair = nullptr;
+                it.second.diffpair_primary = false;
+            }
         }
     }
 }
@@ -218,7 +221,7 @@ void Block::vacuum_nets()
 {
     std::set<UUID> nets_erase;
     for (const auto &it : nets) {
-        if (!it.second.is_power && !it.second.is_port) { // don't vacuum power nets and ports
+        if (!it.second.is_power && !it.second.is_port && !it.second.keep) { // don't vacuum power nets and ports
             nets_erase.emplace(it.first);
         }
     }
@@ -575,29 +578,16 @@ void Block::swap_gates(const UUID &comp_uu, const UUID &g1_uu, const UUID &g2_uu
 ItemSet Block::get_pool_items_used() const
 {
     ItemSet items_needed;
-    auto add_part = [&items_needed](const Part *part) {
-        while (part) {
-            items_needed.emplace(ObjectType::PART, part->uuid);
-            items_needed.emplace(ObjectType::PACKAGE, part->package.uuid);
-            for (const auto &it_pad : part->package->pads) {
-                items_needed.emplace(ObjectType::PADSTACK, it_pad.second.pool_padstack->uuid);
-            }
-            part = part->base;
-        }
-    };
 
     for (const auto &it : components) {
-        items_needed.emplace(ObjectType::ENTITY, it.second.entity->uuid);
-        for (const auto &it_gate : it.second.entity->gates) {
-            items_needed.emplace(ObjectType::UNIT, it_gate.second.unit->uuid);
-        }
-        if (it.second.part) {
-            add_part(it.second.part);
-        }
+        const auto comp_items = it.second.get_pool_items_used();
+        items_needed.insert(comp_items.begin(), comp_items.end());
     }
     for (const auto &it : bom_export_settings.concrete_parts) {
-        add_part(it.second);
+        const auto part_items = it.second->get_pool_items_used();
+        items_needed.insert(part_items.begin(), part_items.end());
     }
+
     return items_needed;
 }
 
