@@ -24,7 +24,7 @@ public:
 
     uuid_ptr<class NetClass> net_class;
     uuid_ptr<Net> diffpair;
-    bool diffpair_master = false;
+    bool diffpair_primary = false;
 
     bool is_port = false;
     Pin::Direction port_direction = Pin::Direction::BIDIRECTIONAL;
@@ -33,6 +33,7 @@ public:
     // not saved
     bool is_power_forced = false;
     bool is_bussed = false;
+    bool keep = false;
     unsigned int n_pins_connected = 0;
     bool has_bus_rippers = false;
     std::vector<UUIDVec> hrefs;
