@@ -11,9 +11,12 @@
 #include "common/junction_util.hpp"
 #include "util/bbox_accumulator.hpp"
 #include "util/polygon_arc_removal_proxy.hpp"
+#include <filesystem>
 
 namespace horizon {
 
+namespace fs = std::filesystem;
+
 Board::StackupLayer::StackupLayer(int l) : layer(l)
 {
 }
@@ -39,16 +42,16 @@ const LutEnumStr<Board::OutputFormat> Board::output_format_lut = {
         {"odb", Board::OutputFormat::ODB},
 };
 
-static const unsigned int app_version = 14;
+static const unsigned int app_version = 17;
 
 unsigned int Board::get_app_version()
 {
     return app_version;
 }
 
-Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
+Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, const std::string &board_dir)
     : uuid(uu), block(&iblock), name(j.at("name").get<std::string>()), version(app_version, j),
-      n_inner_layers(j.value("n_inner_layers", 0))
+      board_directory(board_dir), n_inner_layers(j.value("n_inner_layers", 0))
 {
     Logger::log_info("loading board " + (std::string)uuid, Logger::Domain::BOARD);
     if (j.count("stackup")) {
@@ -156,7 +159,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
         const json &o = j["planes"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(planes, ObjectType::PLANE, std::forward_as_tuple(u, it.value(), *this), Logger::Domain::BOARD);
+            load_and_log(planes, ObjectType::PLANE, std::forward_as_tuple(u, it.value(), this), Logger::Domain::BOARD);
         }
     }
     if (j.count("keepouts")) {
@@ -194,7 +197,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
         const json &o = j["included_boards"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(included_boards, ObjectType::BOARD, std::forward_as_tuple(u, it.value()),
+            load_and_log(included_boards, ObjectType::BOARD, std::forward_as_tuple(u, it.value(), board_directory),
                          Logger::Domain::BOARD);
         }
     }
@@ -317,7 +320,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
 Board Board::new_from_file(const std::string &filename, Block &block, IPool &pool)
 {
     auto j = load_json_from_file(filename);
-    return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool);
+    return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool, fs::u8path(filename).parent_path().u8string());
 }
 
 
@@ -363,7 +366,8 @@ Board::Board(const Board &brd, CopyMode copy_mode)
       gerber_output_settings(brd.gerber_output_settings), odb_output_settings(brd.odb_output_settings),
       grid_settings(brd.grid_settings), airwires(brd.airwires), stackup(brd.stackup), colors(brd.colors),
       pdf_export_settings(brd.pdf_export_settings), step_export_settings(brd.step_export_settings),
-      pnp_export_settings(brd.pnp_export_settings), version(brd.version), n_inner_layers(brd.n_inner_layers)
+      pnp_export_settings(brd.pnp_export_settings), version(brd.version), board_directory(brd.board_directory),
+      n_inner_layers(brd.n_inner_layers)
 {
     if (copy_mode == CopyMode::DEEP) {
         packages = brd.packages;
@@ -893,51 +897,6 @@ void Board::smash_package(BoardPackage *pkg)
     }
 }
 
-void Board::copy_package_silkscreen_texts(BoardPackage *dest, const BoardPackage *src)
-{
-    // Copy smash status
-    unsmash_package(dest);
-    if (!src->smashed) {
-        return;
-    }
-    dest->smashed = true;
-
-    // Copy all texts from src
-    for (const auto &it : src->texts) {
-        if (!BoardLayers::is_silkscreen(it->layer)) {
-            // Not on top or bottom silkscreen
-            continue;
-        }
-
-        auto uu = UUID::random();
-        auto &x = texts.emplace(uu, uu).first->second;
-        x.from_smash = true;
-        x.overridden = true;
-
-        x.placement = dest->placement;
-        Placement rp = it->placement;
-        Placement ref_placement = src->placement;
-        if (ref_placement.mirror) {
-            ref_placement.invert_angle();
-        }
-        rp.make_relative(ref_placement);
-        if (x.placement.mirror) {
-            x.placement.invert_angle();
-        }
-        x.placement.accumulate(rp);
-
-        x.text = it->text;
-        x.layer = it->layer;
-        if (src->flip != dest->flip) {
-            flip_package_layer(x.layer);
-        }
-
-        x.size = it->size;
-        x.width = it->width;
-        dest->texts.push_back(&x);
-    }
-}
-
 void Board::smash_package_silkscreen_graphics(BoardPackage *pkg)
 {
     if (pkg->omit_silkscreen)
@@ -1244,6 +1203,34 @@ json Board::serialize() const
     return j;
 }
 
+
+json Board::serialize_planes() const
+{
+    json j;
+    j["planes"] = json::object();
+    for (const auto &it : planes) {
+        j["planes"][(std::string)it.first] = it.second.serialize_fragments();
+    }
+    return j;
+}
+
+void Board::load_planes(const json &j)
+{
+    if (j.count("planes")) {
+        for (const auto &[uu, it] : j.at("planes").items()) {
+            if (planes.count(uu)) {
+                auto &plane = planes.at(uu);
+                plane.load_fragments(it);
+            }
+        }
+    }
+}
+
+void Board::load_planes_from_file(const std::string &filename)
+{
+    load_planes(load_json_from_file(filename));
+}
+
 void Board::save_pictures(const std::string &dir) const
 {
     pictures_save({&pictures}, dir, "brd");
@@ -1336,6 +1323,20 @@ struct PolyInfo {
     }
 };
 
+static void check_zero_length_edges(Board::Outline &outline, const Polygon &poly)
+{
+    for (size_t i = 0; i < poly.vertices.size(); i++) {
+        const auto p0 = poly.get_vertex(i).position;
+        const auto p1 = poly.get_vertex(i + 1).position;
+        if (p1 == p0) {
+            outline.errors.errors.emplace_back(RulesCheckErrorLevel::WARN, "Zero-length edge");
+            auto &x = outline.errors.errors.back();
+            x.has_location = true;
+            x.location = p0;
+        }
+    }
+};
+
 Board::Outline Board::get_outline(bool with_errors) const
 {
     std::vector<PolyInfo> polys;
@@ -1375,6 +1376,11 @@ Board::Outline Board::get_outline(bool with_errors) const
         if (ClipperLib::Orientation(outline_poly.path))
             outline.outline.reverse();
 
+        if (with_errors) {
+            check_zero_length_edges(outline, outline.outline);
+            outline.errors.update();
+        }
+
         return outline;
     }
 
@@ -1457,6 +1463,14 @@ Board::Outline Board::get_outline(bool with_errors) const
         }
     }
 
+    if (with_errors) {
+        check_zero_length_edges(outline, outline.outline);
+        for (const auto &poly : outline.holes) {
+            check_zero_length_edges(outline, poly);
+        }
+        outline.errors.update();
+    }
+
     return outline;
 }
 
