@@ -54,7 +54,7 @@ private:
     void vacuum_junctions();
 
 public:
-    Board(const UUID &uu, const json &, Block &block, IPool &pool);
+    Board(const UUID &uu, const json &, Block &block, IPool &pool, const std::string &board_directory);
     static Board new_from_file(const std::string &filename, Block &block, IPool &pool);
     Board(const UUID &uu, Block &block);
     static unsigned int get_app_version();
@@ -71,7 +71,6 @@ public:
     void disconnect_package(BoardPackage *pkg);
 
     void smash_package(BoardPackage *pkg);
-    void copy_package_silkscreen_texts(BoardPackage *dest, const BoardPackage *src);
     void unsmash_package(BoardPackage *pkg);
     void smash_package_silkscreen_graphics(BoardPackage *pkg);
     void smash_package_outline(BoardPackage &pkg);
@@ -161,6 +160,11 @@ public:
     std::set<UUID> airwires_expand;
 
     json serialize() const;
+    json serialize_planes() const;
+    void load_planes(const json &j);
+    void load_planes_from_file(const std::string &filename);
+    void save_planes(const std::string &filename);
+
     void save_pictures(const std::string &dir) const;
     void load_pictures(const std::string &dir);
 
@@ -180,6 +184,8 @@ public:
     Outline get_outline() const;
     Outline get_outline_and_errors() const;
 
+    std::string board_directory; // for resolving relative paths in included boards
+
 private:
     unsigned int n_inner_layers = 0;
     ClipperLib::Paths get_thermals(class Plane *plane, const class CanvasPads *ca) const;
