@@ -18,9 +18,10 @@ BoardRules::BoardRules(const BoardRules &other)
       rule_clearance_copper_other(other.rule_clearance_copper_other), rule_plane(other.rule_plane),
       rule_diffpair(other.rule_diffpair), rule_clearance_copper_keepout(other.rule_clearance_copper_keepout),
       rule_layer_pair(other.rule_layer_pair), rule_clearance_same_net(other.rule_clearance_same_net),
-      rule_shorted_pads(other.rule_shorted_pads),
+      rule_shorted_pads(other.rule_shorted_pads), rule_thermals(other.rule_thermals),
       rule_clearance_silkscreen_exposed_copper(other.rule_clearance_silkscreen_exposed_copper),
-      rule_parameters(other.rule_parameters), rule_preflight_checks(other.rule_preflight_checks)
+      rule_parameters(other.rule_parameters), rule_preflight_checks(other.rule_preflight_checks),
+      rule_net_ties(other.rule_net_ties), rule_board_connectivity(other.rule_board_connectivity)
 {
     update_sorted();
 }
@@ -38,9 +39,12 @@ void BoardRules::operator=(const BoardRules &other)
     rule_layer_pair = other.rule_layer_pair;
     rule_clearance_same_net = other.rule_clearance_same_net;
     rule_shorted_pads = other.rule_shorted_pads;
+    rule_thermals = other.rule_thermals;
     rule_clearance_silkscreen_exposed_copper = other.rule_clearance_silkscreen_exposed_copper;
     rule_parameters = other.rule_parameters;
     rule_preflight_checks = other.rule_preflight_checks;
+    rule_net_ties = other.rule_net_ties;
+    rule_board_connectivity = other.rule_board_connectivity;
 
     update_sorted();
 }
@@ -160,6 +164,13 @@ void BoardRules::import_rules(const json &j, const RuleImportMap &import_map)
         }
         fix_order(RuleID::SHORTED_PADS);
     }
+    if (j.count("thermals")) {
+        for (const auto &[key, value] : j.at("thermals").items()) {
+            UUID u = key;
+            rule_thermals.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(u, value));
+        }
+        fix_order(RuleID::THERMALS);
+    }
     if (j.count("clearance_silkscreen_exposed_copper")) {
         const json &o = j["clearance_silkscreen_exposed_copper"];
         rule_clearance_silkscreen_exposed_copper = RuleClearanceSilkscreenExposedCopper(o, import_map);
@@ -202,8 +213,13 @@ void BoardRules::cleanup(const Block *block)
         it.second.match.cleanup(block);
     }
     for (auto &it : rule_shorted_pads) {
+        it.second.match_component.cleanup(block);
         it.second.match.cleanup(block);
     }
+    for (auto &it : rule_thermals) {
+        it.second.match.cleanup(block);
+        it.second.match_component.cleanup(block);
+    }
     for (auto &it : rule_diffpair) {
         if (!block->net_classes.count(it.second.net_class))
             it.second.net_class = block->net_class_default->uuid;
@@ -220,6 +236,12 @@ void BoardRules::apply(RuleID id, Board &brd, IPool &pool) const
                 track.width = get_default_track_width(track.net, track.layer);
             }
         }
+        for (auto &it : brd.net_ties) {
+            auto &tie = it.second;
+            if (tie.width_from_rules) {
+                tie.width = get_default_track_width(tie.net_tie->net_primary, tie.layer);
+            }
+        }
     }
     else if (id == RuleID::PARAMETERS) {
         brd.rules.rule_parameters = rule_parameters;
@@ -274,6 +296,7 @@ json BoardRules::serialize_or_export(Rule::SerializeMode mode) const
     SERIALIZE(plane);
     SERIALIZE(diffpair);
     SERIALIZE(shorted_pads);
+    SERIALIZE(thermals);
     SERIALIZE(clearance_copper_other);
     SERIALIZE(clearance_copper_keepout);
     SERIALIZE(clearance_same_net);
@@ -305,12 +328,15 @@ std::vector<RuleID> BoardRules::get_rule_ids() const
 
             RuleID::VIA,
             RuleID::PLANE,
+            RuleID::THERMALS,
             RuleID::DIFFPAIR,
             RuleID::PARAMETERS,
 
             RuleID::SHORTED_PADS,
+            RuleID::NET_TIES,
             RuleID::LAYER_PAIR,
             RuleID::PREFLIGHT_CHECKS,
+            RuleID::BOARD_CONNECTIVITY,
     };
 }
 
@@ -325,6 +351,12 @@ const Rule &BoardRules::get_rule(RuleID id) const
     else if (id == RuleID::PREFLIGHT_CHECKS) {
         return rule_preflight_checks;
     }
+    else if (id == RuleID::NET_TIES) {
+        return rule_net_ties;
+    }
+    else if (id == RuleID::BOARD_CONNECTIVITY) {
+        return rule_board_connectivity;
+    }
     throw std::runtime_error("rule does not exist");
 }
 
@@ -353,6 +385,8 @@ const Rule &BoardRules::get_rule(RuleID id, const UUID &uu) const
         return rule_clearance_same_net.at(uu);
     case RuleID::SHORTED_PADS:
         return rule_shorted_pads.at(uu);
+    case RuleID::THERMALS:
+        return rule_thermals.at(uu);
     default:
         throw std::runtime_error("rule does not exist");
     }
@@ -426,6 +460,12 @@ std::map<UUID, const Rule *> BoardRules::get_rules(RuleID id) const
         }
         break;
 
+    case RuleID::THERMALS:
+        for (auto &it : rule_thermals) {
+            r.emplace(it.first, &it.second);
+        }
+        break;
+
     default:;
     }
     return r;
@@ -478,6 +518,10 @@ void BoardRules::remove_rule(RuleID id, const UUID &uu)
         rule_shorted_pads.erase(uu);
         break;
 
+    case RuleID::THERMALS:
+        rule_thermals.erase(uu);
+        break;
+
     default:;
     }
     fix_order(id);
@@ -533,6 +577,10 @@ Rule &BoardRules::add_rule(RuleID id)
         r = &rule_shorted_pads.emplace(uu, uu).first->second;
         break;
 
+    case RuleID::THERMALS:
+        r = &rule_thermals.emplace(uu, uu).first->second;
+        break;
+
     default:
         throw std::runtime_error("unsupported rule");
     }
@@ -699,6 +747,23 @@ const PlaneSettings &BoardRules::get_plane_settings(const Net *net, int layer) c
     return plane_settings_default;
 }
 
+const ThermalSettings &BoardRules::get_thermal_settings(const Plane &plane, const BoardPackage &pkg,
+                                                        const Pad &pad) const
+{
+    const RuleThermals *rule = nullptr;
+    for (const auto it : get_rules_sorted<RuleThermals>()) {
+        if (it->matches(pkg, pad, plane.polygon->layer)) {
+            rule = it;
+            break;
+        }
+    }
+
+    if (rule && rule->thermal_settings.connect_style != ThermalSettings::ConnectStyle::FROM_PLANE)
+        return rule->thermal_settings;
+    else
+        return plane.settings.thermal_settings;
+}
+
 int BoardRules::get_layer_pair(const Net *net, int layer) const
 {
     auto rules = get_rules_sorted<RuleLayerPair>();
