@@ -14,8 +14,12 @@
 #include "rule_layer_pair.hpp"
 #include "rule_clearance_same_net.hpp"
 #include "rule_shorted_pads.hpp"
+#include "rule_thermals.hpp"
+#include "rule_net_ties.hpp"
+#include "rule_board_connectivity.hpp"
 #include "rules/rules.hpp"
 #include "util/uuid.hpp"
+#include <atomic>
 
 namespace horizon {
 using json = nlohmann::json;
@@ -27,8 +31,8 @@ public:
     void operator=(const BoardRules &other);
 
     void load_from_json(const json &j) override;
-    RulesCheckResult check(RuleID id, const class Board &b, class RulesCheckCache &cache,
-                           check_status_cb_t status_cb) const;
+    RulesCheckResult check(RuleID id, const class Board &b, class RulesCheckCache &cache, check_status_cb_t status_cb,
+                           const std::atomic_bool &cancel = std::atomic_bool(false)) const;
     void apply(RuleID id, class Board &b, class IPool &pool) const;
     json serialize() const override;
     std::vector<RuleID> get_rule_ids() const override;
@@ -54,6 +58,8 @@ public:
 
     const PlaneSettings &get_plane_settings(const class Net *net, int layer) const;
 
+    const ThermalSettings &get_thermal_settings(const Plane &plane, const BoardPackage &pkg, const Pad &pad) const;
+
     int get_layer_pair(const Net *net, int layer) const;
 
     json export_rules(const class RulesExportInfo &export_info, const Board &brd) const;
@@ -76,6 +82,7 @@ private:
     std::map<UUID, RuleLayerPair> rule_layer_pair;
     std::map<UUID, RuleClearanceSameNet> rule_clearance_same_net;
     std::map<UUID, RuleShortedPads> rule_shorted_pads;
+    std::map<UUID, RuleThermals> rule_thermals;
 
     std::vector<const RuleClearanceCopper *> rule_sorted_clearance_copper;
     void update_sorted();
@@ -83,21 +90,30 @@ private:
     RuleClearanceSilkscreenExposedCopper rule_clearance_silkscreen_exposed_copper;
     RuleParameters rule_parameters;
     RulePreflightChecks rule_preflight_checks;
+    RuleNetTies rule_net_ties;
+    RuleBoardConnectivity rule_board_connectivity;
+
 
     RulesCheckResult check_track_width(const class Board &b) const;
     RulesCheckResult check_hole_size(const class Board &b) const;
     RulesCheckResult check_clearance_copper(const class Board &b, class RulesCheckCache &cache,
-                                            check_status_cb_t status_cb) const;
+                                            check_status_cb_t status_cb, const std::atomic_bool &cancel) const;
     RulesCheckResult check_clearance_copper_non_copper(const class Board &b, class RulesCheckCache &cache,
-                                                       check_status_cb_t status_cb) const;
+                                                       check_status_cb_t status_cb,
+                                                       const std::atomic_bool &cancel) const;
     RulesCheckResult check_clearance_silkscreen_exposed_copper(const class Board &b, class RulesCheckCache &cache,
-                                                               check_status_cb_t status_cb) const;
+                                                               check_status_cb_t status_cb,
+                                                               const std::atomic_bool &cancel) const;
     RulesCheckResult check_preflight(const class Board &b) const;
     RulesCheckResult check_clearance_copper_keepout(const class Board &b, class RulesCheckCache &cache,
-                                                    check_status_cb_t status_cb) const;
+                                                    check_status_cb_t status_cb, const std::atomic_bool &cancel) const;
     RulesCheckResult check_clearance_same_net(const class Board &b, class RulesCheckCache &cache,
-                                              check_status_cb_t status_cb) const;
+                                              check_status_cb_t status_cb, const std::atomic_bool &cancel) const;
     RulesCheckResult check_plane_priorities(const class Board &b) const;
+    RulesCheckResult check_net_ties(const class Board &b, RulesCheckCache &cache, check_status_cb_t status_cb,
+                                    const std::atomic_bool &cancel) const;
+    RulesCheckResult check_board_connectivity(const class Board &b, RulesCheckCache &cache, check_status_cb_t status_cb,
+                                              const std::atomic_bool &cancel) const;
 
     json serialize_or_export(Rule::SerializeMode mode) const;
 };
