@@ -77,11 +77,14 @@ json PlaneSettings::serialize() const
     return j;
 }
 
-Plane::Plane(const UUID &uu, const json &j, Board &brd)
-    : uuid(uu), net(&brd.block->nets.at(j.at("net").get<std::string>())),
-      polygon(&brd.polygons.at(j.at("polygon").get<std::string>())), from_rules(j.value("from_rules", true)),
-      priority(j.value("priority", 0))
+Plane::Plane(const UUID &uu, const json &j, Board *brd)
+    : uuid(uu), net(j.at("net").get<std::string>()), polygon(j.at("polygon").get<std::string>()),
+      from_rules(j.value("from_rules", true)), priority(j.value("priority", 0))
 {
+    if (brd) {
+        net.update(brd->block->nets);
+        polygon.update(brd->polygons);
+    }
     if (j.count("settings")) {
         settings = PlaneSettings(j.at("settings"));
     }
@@ -114,10 +117,27 @@ json Plane::serialize() const
     j["priority"] = priority;
     j["from_rules"] = from_rules;
     j["settings"] = settings.serialize();
+    return j;
+}
+
+void Plane::load_fragments(const json &j)
+{
+    if (j.count("fragments")) {
+        fragments.clear();
+        for (const auto &it : j.at("fragments")) {
+            fragments.emplace_back(it);
+        }
+        revision++;
+    }
+}
+
+json Plane::serialize_fragments() const
+{
     auto frags = json::array();
     for (const auto &it : fragments) {
         frags.push_back(it.serialize());
     }
+    json j;
     j["fragments"] = frags;
     return j;
 }
