@@ -65,7 +65,7 @@ public:
         json serialize() const;
     };
 
-    Plane(const UUID &uu, const json &j, class Board &brd);
+    Plane(const UUID &uu, const json &j, class Board *brd);
     Plane(const UUID &uu);
     UUID uuid;
     uuid_ptr<Net> net;
@@ -80,12 +80,14 @@ public:
     {
         return revision;
     }
+    void load_fragments(const json &j);
 
     Type get_type() const override;
     UUID get_uuid() const override;
     std::string get_name() const;
 
     json serialize() const;
+    json serialize_fragments() const;
 
 private:
     unsigned int revision = 0;
