@@ -97,7 +97,8 @@ static bool bb_isect(const std::pair<ClipperLib::IntPoint, ClipperLib::IntPoint>
            && (bb1.second.Y >= bb2.first.Y && bb2.second.Y >= bb1.first.Y);
 }
 
-void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPads *ca_pads_ext)
+void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPads *ca_pads_ext,
+                         plane_update_status_cb_t status_cb, const std::atomic_bool &cancel)
 {
     MyCanvasPatch ca_my;
     const CanvasPatch *ca = ca_ext;
@@ -131,7 +132,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
     }
     const double twiddle = .005_mm;
     ClipperLib::Paths out; // the plane before expanding by min_width
-
+    if (status_cb)
+        status_cb(*plane, "Starting…");
 
     {
         ClipperLib::Clipper cl_plane;
@@ -144,7 +146,11 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
         auto poly_bb = get_path_bb(poly_path);
 
+        if (cancel)
+            return;
 
+        if (status_cb)
+            status_cb(*plane, "Shrinking outline…");
         {
             ClipperLib::ClipperOffset ofs_poly; // shrink polygon contour by min_width/2
             ofs_poly.ArcTolerance = 2e3;
@@ -154,14 +160,16 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
             cl_plane.AddPaths(poly_shrink, ClipperLib::ptSubject, true);
         }
 
-
+        if (status_cb)
+            status_cb(*plane, "Adding cutouts…");
         for (const auto &patch : ca->get_patches()) { // add cutouts
             if ((patch.first.layer == poly.layer && patch.first.net != plane->net->uuid && patch.second.size()
                  && patch.first.type != PatchType::OTHER && patch.first.type != PatchType::TEXT)
                 || ((patch.first.layer == 10000)
                     && ((patch.first.type == PatchType::HOLE_NPTH)
                         || ((patch.first.type == PatchType::HOLE_PTH) && (patch.first.net != plane->net->uuid))))) {
-
+                if (cancel)
+                    return;
                 int64_t clearance = 0;
                 if (patch.first.type != PatchType::HOLE_NPTH) { // copper
                     auto patch_net = patch.first.net ? &block->nets.at(patch.first.net) : nullptr;
@@ -197,7 +205,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
         auto text_clearance = rules.get_clearance_copper_other(plane->net, poly.layer)
                                       ->get_clearance(PatchType::PLANE, PatchType::TEXT);
-
+        if (status_cb)
+            status_cb(*plane, "Adding text cutouts…");
         // add text cutouts
         if (plane->settings.text_style == PlaneSettings::TextStyle::EXPAND) {
             for (const auto &patch : ca->get_patches()) { // add cutouts
@@ -235,6 +244,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
             }
         }
 
+        if (status_cb)
+            status_cb(*plane, "Adding keepouts…");
         // add keepouts
         {
             auto keepout_rules = rules.get_rules_sorted<RuleClearanceCopperKeepout>();
@@ -243,6 +254,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
                 const auto keepout = it_keepout.keepout;
                 if ((plane->polygon->layer == keepout->polygon->layer || keepout->all_cu_layers)
                     && keepout->patch_types_cu.count(PatchType::PLANE)) {
+                    if (cancel)
+                        return;
                     auto clearance = rules.get_clearance_copper_keepout(plane->net, &it_keepout)
                                              ->get_clearance(PatchType::PLANE);
 
@@ -259,6 +272,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
         cl_plane.Execute(ClipperLib::ctDifference, out, ClipperLib::pftNonZero); // do cutouts
     }
 
+    if (status_cb)
+        status_cb(*plane, "Clipping to board outline…");
     // do board outline clearance
     CanvasPatch::PatchKey outline_key;
     outline_key.layer = BoardLayers::L_OUTLINE;
@@ -300,29 +315,39 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
     ClipperLib::PolyTree tree;
 
 
-    if (plane->settings.connect_style == PlaneSettings::ConnectStyle::THERMAL) {
+    {
         ClipperLib::Paths plane_with_thermal_cutouts;
         {
+            if (status_cb)
+                status_cb(*plane, "Adding thermal cutouts …");
             ClipperLib::Clipper cl_plane;
             cl_plane.AddPaths(out, ClipperLib::ptSubject, true);
 
             // add thermal coutouts
-            for (const auto &patch : ca->get_patches()) {
-                if ((patch.first.layer == poly.layer && patch.first.net == plane->net->uuid
-                     && (patch.first.type == PatchType::PAD || patch.first.type == PatchType::PAD_TH))) {
-                    ClipperLib::ClipperOffset ofs; // expand patch for cutout
-                    ofs.ArcTolerance = 2e3;
-                    ClipperLib::Paths pad;
-                    ClipperLib::SimplifyPolygons(patch.second, pad, ClipperLib::pftNonZero);
+            for (const auto &[key, paths] : ca_pads->pads) {
+                const auto &pkg = packages.at(key.package);
+                const auto &pkg_pad = pkg.package.pads.at(key.pad);
+                const Net *pad_net = pkg_pad.net;
+                if (key.layer == poly.layer && (pad_net && (pad_net->uuid == plane->net->uuid))) {
+                    if (const auto &th = rules.get_thermal_settings(*plane, pkg, pkg_pad);
+                        th.connect_style == ThermalSettings::ConnectStyle::THERMAL) {
+                        if (cancel)
+                            return;
 
-                    ofs.AddPaths(pad, ClipperLib::jtMiter, ClipperLib::etClosedPolygon);
-                    ClipperLib::Paths patch_exp;
+                        ClipperLib::ClipperOffset ofs; // expand patch for cutout
+                        ofs.ArcTolerance = 2e3;
+                        ClipperLib::Paths pad;
+                        ClipperLib::SimplifyPolygons(paths.second, pad, ClipperLib::pftNonZero);
 
-                    double expand = plane->settings.thermal_gap_width + plane->settings.min_width / 2;
-                    ofs.Execute(patch_exp, expand);
+                        ofs.AddPaths(pad, ClipperLib::jtMiter, ClipperLib::etClosedPolygon);
+                        ClipperLib::Paths patch_exp;
+
+                        double expand = th.thermal_gap_width + plane->settings.min_width / 2;
+                        ofs.Execute(patch_exp, expand);
 
 
-                    cl_plane.AddPaths(patch_exp, ClipperLib::ptClip, true);
+                        cl_plane.AddPaths(patch_exp, ClipperLib::ptClip, true);
+                    }
                 }
             }
 
@@ -331,10 +356,14 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
         ClipperLib::Paths before_expand;
         {
+            if (status_cb)
+                status_cb(*plane, "Adding thermal spokes …");
             ClipperLib::Clipper cl_add_spokes;
             cl_add_spokes.AddPaths(plane_with_thermal_cutouts, ClipperLib::ptSubject, true);
             auto thermals = get_thermals(plane, ca_pads);
             for (const auto &spoke : thermals) {
+                if (cancel)
+                    return;
                 ClipperLib::Paths test_result;
                 ClipperLib::Clipper cl_test;
                 cl_test.AddPaths(plane_with_thermal_cutouts, ClipperLib::ptSubject, true);
@@ -350,6 +379,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
         ClipperLib::Paths clipped_to_orig;
         {
+            if (status_cb)
+                status_cb(*plane, "Clipping thermal spokes…");
             ClipperLib::Clipper cl_clip_to_orig;
             cl_clip_to_orig.AddPaths(before_expand, ClipperLib::ptSubject, true);
             cl_clip_to_orig.AddPaths(out, ClipperLib::ptClip, true);
@@ -358,6 +389,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
 
         {
+            if (status_cb)
+                status_cb(*plane, "Expanding for minimum width…");
             ClipperLib::ClipperOffset ofs;
             ofs.ArcTolerance = 2e3;
             ofs.AddPaths(clipped_to_orig, jt, ClipperLib::etClosedPolygon);
@@ -369,19 +402,11 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
             }
         }
     }
-    else {
-        ClipperLib::ClipperOffset ofs;
-        ofs.ArcTolerance = 2e3;
-        ofs.AddPaths(out, jt, ClipperLib::etClosedPolygon);
-        if (plane->settings.fill_style == PlaneSettings::FillStyle::SOLID) {
-            ofs.Execute(tree, plane->settings.min_width / 2);
-        }
-        else {
-            ofs.Execute(final, plane->settings.min_width / 2);
-        }
-    }
+
 
     if (plane->settings.fill_style == PlaneSettings::FillStyle::HATCH) {
+        if (status_cb)
+            status_cb(*plane, "Computing hatch…");
         ClipperLib::Paths contracted;
         {
             ClipperLib::ClipperOffset cl;
@@ -440,11 +465,13 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
             cl.Execute(ClipperLib::ctUnion, tree, ClipperLib::pftNonZero);
         }
     }
-
+    if (status_cb)
+        status_cb(*plane, "Creating fragments…");
     for (const auto node : tree.Childs) {
         polynode_to_fragment(plane, node);
     }
-
+    if (status_cb)
+        status_cb(*plane, "Finding orphans…");
     // orphan all
     for (auto &it : plane->fragments) {
         it.orphan = true;
@@ -452,6 +479,8 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
 
     // find orphans
     for (auto &frag : plane->fragments) {
+        if (cancel)
+            return;
         for (const auto &it : junctions) {
             if (it.second.net == plane->net && (it.second.layer == plane->polygon->layer || it.second.has_via)
                 && frag.contains(it.second.position)) {
@@ -490,10 +519,13 @@ void Board::update_plane(Plane *plane, const CanvasPatch *ca_ext, const CanvasPa
                                               [](const auto &x) { return x.orphan; }),
                                plane->fragments.end());
     }
+    if (status_cb)
+        status_cb(*plane, "Done");
 }
 
 static void plane_update_worker(std::mutex &mutex, std::set<Plane *> &planes, Board *brd, const CanvasPatch *ca_patch,
-                                const CanvasPads *ca_pads)
+                                const CanvasPads *ca_pads, plane_update_status_cb_t status_cb,
+                                const std::atomic_bool &cancel)
 {
     while (true) {
         Plane *plane = nullptr;
@@ -506,11 +538,11 @@ static void plane_update_worker(std::mutex &mutex, std::set<Plane *> &planes, Bo
             planes.erase(it);
         }
         assert(plane);
-        brd->update_plane(plane, ca_patch, ca_pads);
+        brd->update_plane(plane, ca_patch, ca_pads, status_cb, cancel);
     }
 }
 
-void Board::update_planes()
+void Board::update_planes(plane_update_status_cb_t status_cb, const std::atomic_bool &cancel)
 {
     std::map<int, std::set<Plane *>> planes_by_priority;
     std::set<UUID> nets;
@@ -534,21 +566,19 @@ void Board::update_planes()
         std::mutex plane_update_mutex;
         std::set<Plane *> planes_for_workers = planes_by_priority.at(priority);
 
-        if (planes_for_workers.size() >= std::thread::hardware_concurrency()) {
+        {
             std::vector<std::thread> threads;
-            for (size_t i = 0; i < std::thread::hardware_concurrency(); i++) {
+            const auto n_threads =
+                    std::min(static_cast<size_t>(std::thread::hardware_concurrency()), planes_for_workers.size());
+            for (size_t i = 0; i < n_threads; i++) {
                 threads.emplace_back(plane_update_worker, std::ref(plane_update_mutex), std::ref(planes_for_workers),
-                                     this, &ca, &cp);
+                                     this, &ca, &cp, status_cb, std::ref(cancel));
             }
             for (auto &thr : threads) {
                 thr.join();
             }
         }
-        else {
-            for (auto plane : planes_by_priority.at(priority)) {
-                update_plane(plane, &ca, &cp);
-            }
-        }
+
         for (auto plane : planes_by_priority.at(priority)) {
             ca.append_polygon(*(plane->polygon));
         }
@@ -563,10 +593,16 @@ ClipperLib::Paths Board::get_thermals(Plane *plane, const CanvasPads *cp) const
         if (it.first.layer != plane->polygon->layer)
             continue;
 
-        auto net_from_pad = packages.at(it.first.package).package.pads.at(it.first.pad).net;
+        const auto &pkg = packages.at(it.first.package);
+        const auto &pad = pkg.package.pads.at(it.first.pad);
+        auto net_from_pad = pad.net;
         if (net_from_pad != plane->net)
             continue;
 
+        const auto &th = rules.get_thermal_settings(*plane, pkg, pad);
+        if (th.connect_style == ThermalSettings::ConnectStyle::SOLID)
+            continue;
+
         ClipperLib::Paths uni; // union of all off the pads polygons
         ClipperLib::SimplifyPolygons(it.second.second, uni, ClipperLib::pftNonZero);
 
@@ -575,7 +611,7 @@ ClipperLib::Paths Board::get_thermals(Plane *plane, const CanvasPads *cp) const
         ofs.AddPaths(uni, ClipperLib::jtMiter, ClipperLib::etClosedPolygon);
         ClipperLib::Paths pad_exp; // pad expanded by gap width
 
-        double expand = plane->settings.thermal_gap_width + .01_mm + plane->settings.min_width / 2;
+        double expand = th.thermal_gap_width + .01_mm + plane->settings.min_width / 2;
         ofs.Execute(pad_exp, expand);
 
         auto p0 = pad_exp.front().front();
@@ -592,8 +628,7 @@ ClipperLib::Paths Board::get_thermals(Plane *plane, const CanvasPads *cp) const
         auto h = bb.second.y - bb.first.y;
         auto l = std::max(w, h);
 
-        int64_t spoke_width =
-                std::max(.01_mm, (int64_t)plane->settings.thermal_spoke_width - (int64_t)plane->settings.min_width);
+        int64_t spoke_width = std::max(.01_mm, (int64_t)th.thermal_spoke_width - (int64_t)plane->settings.min_width);
         ClipperLib::Path spoke;
         spoke.emplace_back(-spoke_width / 2, -spoke_width / 2);
         spoke.emplace_back(-spoke_width / 2, spoke_width / 2);
@@ -603,13 +638,13 @@ ClipperLib::Paths Board::get_thermals(Plane *plane, const CanvasPads *cp) const
         ClipperLib::Paths antipad;
         {
 
-            for (int angle = 0; angle < 360; angle += 90) {
+            for (unsigned int i = 0; i < th.n_spokes; i++) {
                 ClipperLib::Clipper cl;
                 cl.AddPaths(pad_exp, ClipperLib::ptSubject, true);
 
                 ClipperLib::Path r1 = spoke;
                 Placement tr;
-                tr.set_angle_deg(angle);
+                tr.set_angle((65536 * i) / th.n_spokes + th.angle);
                 transform_path(r1, tr);
                 transform_path(r1, it.second.first);
                 cl.AddPath(r1, ClipperLib::ptClip, true);
