@@ -7,14 +7,12 @@
 namespace horizon {
 RuleDiffpair::RuleDiffpair(const UUID &uu) : Rule(uu)
 {
-    id = RuleID::DIFFPAIR;
 }
 
 RuleDiffpair::RuleDiffpair(const UUID &uu, const json &j, const RuleImportMap &import_map)
     : Rule(uu, j, import_map), net_class(import_map.get_net_class(j.at("net_class").get<std::string>())),
       layer(j.at("layer")), track_width(j.at("track_width")), track_gap(j.at("track_gap")), via_gap(j.at("via_gap"))
 {
-    id = RuleID::DIFFPAIR;
 }
 
 json RuleDiffpair::serialize() const
@@ -28,8 +26,8 @@ json RuleDiffpair::serialize() const
     return j;
 }
 
-std::string RuleDiffpair::get_brief(const class Block *block) const
+std::string RuleDiffpair::get_brief(const class Block *block, class IPool *pool) const
 {
-    return "Net class " + (net_class ? block->net_classes.at(net_class).name : "?");
+    return "Net class " + (net_class ? block->net_classes.at(net_class).name : "?") + "\n" + layer_to_string(layer);
 }
 } // namespace horizon
