@@ -6,11 +6,17 @@
 namespace horizon {
 class RuleDiffpair : public Rule {
 public:
+    static const auto id = RuleID::DIFFPAIR;
+    RuleID get_id() const override
+    {
+        return id;
+    }
+
     RuleDiffpair(const UUID &uu);
     RuleDiffpair(const UUID &uu, const json &j, const RuleImportMap &import_map);
     json serialize() const override;
 
-    std::string get_brief(const class Block *block = nullptr) const override;
+    std::string get_brief(const class Block *block = nullptr, class IPool *pool = nullptr) const override;
     bool can_export() const override
     {
         return true;
