@@ -5,11 +5,17 @@
 namespace horizon {
 class RuleParameters : public Rule {
 public:
+    static const auto id = RuleID::PARAMETERS;
+    RuleID get_id() const override
+    {
+        return id;
+    }
+
     RuleParameters();
     RuleParameters(const json &j, const RuleImportMap &import_map);
     json serialize() const override;
 
-    std::string get_brief(const class Block *block = nullptr) const override;
+    std::string get_brief(const class Block *block = nullptr, class IPool *pool = nullptr) const override;
     bool can_export() const override
     {
         return true;
