@@ -3,27 +3,20 @@
 #include "parameter/set.hpp"
 #include "nlohmann/json.hpp"
 #include "board/board_layers.hpp"
+#include "pool/ipool.hpp"
 
 namespace horizon {
 
-Via::Via(const UUID &uu, const json &j, Board *brd, ViaPadstackProvider *vpp)
-    : uuid(uu), padstack(UUID()), parameter_set(parameter_set_from_json(j.at("parameter_set"))),
-      from_rules(j.at("from_rules")), locked(j.value("locked", false))
+Via::Via(const UUID &uu, const json &j, IPool &pool, Board *brd)
+    : uuid(uu), pool_padstack(pool.get_padstack(j.at("padstack").get<std::string>())), padstack(*pool_padstack),
+      parameter_set(parameter_set_from_json(j.at("parameter_set"))), from_rules(j.at("from_rules")),
+      locked(j.value("locked", false))
 {
     if (brd)
         junction = &brd->junctions.at(j.at("junction").get<std::string>());
     else
         junction.uuid = j.at("junction").get<std::string>();
 
-    if (vpp) {
-        vpp_padstack = vpp->get_padstack(j.at("padstack").get<std::string>());
-        padstack = *vpp_padstack;
-    }
-    else {
-        vpp_padstack.uuid = j.at("padstack").get<std::string>();
-    }
-
-
     if (j.count("net_set")) {
         if (brd)
             net_set = &brd->block->nets.at(j.at("net_set").get<std::string>());
@@ -32,7 +25,7 @@ Via::Via(const UUID &uu, const json &j, Board *brd, ViaPadstackProvider *vpp)
     }
 }
 
-Via::Via(const UUID &uu, const Padstack *ps) : uuid(uu), vpp_padstack(ps), padstack(*vpp_padstack)
+Via::Via(const UUID &uu, const Padstack *ps) : uuid(uu), pool_padstack(ps), padstack(*pool_padstack)
 {
     parameter_set[ParameterID::VIA_DIAMETER] = .5_mm;
     parameter_set[ParameterID::HOLE_DIAMETER] = .2_mm;
@@ -40,7 +33,7 @@ Via::Via(const UUID &uu, const Padstack *ps) : uuid(uu), vpp_padstack(ps), padst
 }
 
 Via::Via(shallow_copy_t sh, const Via &other)
-    : uuid(other.uuid), net_set(other.net_set), junction(other.junction), vpp_padstack(other.vpp_padstack),
+    : uuid(other.uuid), net_set(other.net_set), junction(other.junction), pool_padstack(other.pool_padstack),
       padstack(other.padstack.uuid), parameter_set(other.parameter_set), from_rules(other.from_rules),
       locked(other.locked)
 {
@@ -48,7 +41,7 @@ Via::Via(shallow_copy_t sh, const Via &other)
 
 void Via::expand(const Board &brd)
 {
-    padstack = *vpp_padstack;
+    padstack = *pool_padstack;
     ParameterSet ps_via = parameter_set;
     ps_via.emplace(ParameterID::VIA_SOLDER_MASK_EXPANSION, brd.rules.get_parameters()->via_solder_mask_expansion);
     padstack.apply_parameter_set(ps_via);
@@ -59,7 +52,7 @@ json Via::serialize() const
 {
     json j;
     j["junction"] = (std::string)junction->uuid;
-    j["padstack"] = (std::string)vpp_padstack->uuid;
+    j["padstack"] = (std::string)pool_padstack->uuid;
     j["parameter_set"] = parameter_set_serialize(parameter_set);
     j["from_rules"] = from_rules;
     j["locked"] = locked;
