@@ -45,6 +45,8 @@ public:
     void update(const class BlockSymbol &sym, bool edit = true);
 
     ObjectRef add_line(const std::deque<Coordi> &pts, int64_t width, ColorP color, int layer);
+    ObjectRef add_arc(const Coordi &from, const Coordi &to, const Coordi &center, int64_t width, ColorP color,
+                      int layer);
     void remove_obj(const ObjectRef &r);
     void hide_obj(const ObjectRef &r);
     void show_obj(const ObjectRef &r);
@@ -71,6 +73,8 @@ public:
     bool show_text_in_tracks = false;
     bool show_text_in_vias = false;
 
+    bool add_pad_bbox_targets = false;
+
     virtual bool get_flip_view() const
     {
         return false;
@@ -162,13 +166,10 @@ protected:
                    bool tr = true, uint64_t width = 0);
     void draw_box(const Coord<float> &o, float size, ColorP color = ColorP::FROM_LAYER, int layer = 10000,
                   bool tr = true, uint64_t width = 0);
-    void draw_arc(const Coord<float> &center, float radius, float a0, float a1, ColorP color = ColorP::FROM_LAYER,
-                  int layer = 10000);
     void draw_circle(const Coord<float> &center, float radius, ColorP color = ColorP::FROM_LAYER, int layer = 10000);
-    void draw_arc2(const Coord<float> &center, float radius0, float a0, float a1, ColorP color, int layer,
-                   uint64_t width);
     void draw_arc0(const Coord<float> &center, float radius0, float a0, float a1, ColorP color, int layer,
                    uint64_t width);
+    void draw_arc(const Coordf &from, const Coordf &to, const Coordf &center, ColorP color, int layer, uint64_t width);
 
 
     std::pair<Coordf, Coordf> draw_text(const Coordf &p, float size, const std::string &rtext, int angle,
@@ -214,11 +215,7 @@ protected:
     {
     }
     virtual void img_line(const Coordi &p0, const Coordi &p1, const uint64_t width, int layer = 10000, bool tr = true);
-    virtual void img_arc(const Arc &arc);
-    virtual bool img_supports_arc() const
-    {
-        return false;
-    }
+    virtual void img_arc(const Coordi &from, const Coordi &to, const Coordi &center, const uint64_t width, int layer);
     virtual void img_hole(const Hole &hole)
     {
     }
