@@ -4,6 +4,7 @@ namespace horizon {
 enum class ColorP {
     FROM_LAYER,
     LAYER_HIGHLIGHT,
+    LAYER_HIGHLIGHT_LIGHTEN,
     JUNCTION,
     FRAG_ORPHAN,
     AIRWIRE_ROUTER,
@@ -23,6 +24,7 @@ enum class ColorP {
     NOPOPULATE_X,
     PROJECTION,
     NET_TIE,
+    SYMBOL_BOUNDING_BOX,
     N_COLORS,
     // colors after N_COLORS aren't part of the UBO
     BACKGROUND,
