@@ -215,6 +215,18 @@ ObjectRef Canvas::add_line(const std::deque<Coordi> &pts, int64_t width, ColorP
     return sr;
 }
 
+ObjectRef Canvas::add_arc(const Coordi &from, const Coordi &to, const Coordi &center, int64_t width, ColorP color,
+                          int layer)
+{
+    auto uu = UUID::random();
+    ObjectRef sr(ObjectType::ARC, uu);
+    object_ref_push(sr);
+    draw_arc(from, to, center, color, layer, width);
+    object_ref_pop();
+    request_push();
+    return sr;
+}
+
 void Canvas::render(const Line &line, bool interactive, ColorP co)
 {
     img_line(line.from->position, line.to->position, line.width, line.layer);
@@ -289,12 +301,9 @@ void Canvas::render(const SchematicNetTie &tie)
     const auto r = (4 * h * h + s * s) / (8 * h);
     const auto d = r - h;
     const auto arc_center = center + (perp_n * d);
-    const float a0 = c2pi(atan2f(p0.y - arc_center.y, p0.x - arc_center.x));
-    const float a1 = c2pi(atan2f(p1.y - arc_center.y, p1.x - arc_center.x));
-    // img_auto_line = img_mode;
-    draw_arc2(arc_center, r, a1, a0, ColorP::NET_TIE, 0, 0);
-    draw_arc2(center - (perp_n * d), r, a1 + M_PI, a0 + M_PI, ColorP::NET_TIE, 0, 0);
-
+    img_auto_line = img_mode;
+    draw_arc(p1, p0, arc_center, ColorP::NET_TIE, 0, 0);
+    draw_arc(p0, p1, center - (perp_n * d), ColorP::NET_TIE, 0, 0);
 
     const auto text_pos = center + (perp_n * 1.5_mm);
 
@@ -302,8 +311,8 @@ void Canvas::render(const SchematicNetTie &tie)
     opts.center = true;
     draw_text(text_pos, 1.5_mm,
               get_name(tie.net_tie->net_primary->name) + "\n" + get_name(tie.net_tie->net_secondary->name),
-              angle_from_rad(atan2(v.y, v.x)), TextOrigin::CENTER, ColorP::NET_TIE, 0, opts);
-    // img_auto_line = false;
+              angle_from_rad(v.angle()), TextOrigin::CENTER, ColorP::NET_TIE, 0, opts);
+    img_auto_line = false;
     object_ref_pop();
     if (img_mode)
         return;
@@ -312,16 +321,21 @@ void Canvas::render(const SchematicNetTie &tie)
 
 void Canvas::render(const Track &track, bool interactive)
 {
-    auto c = ColorP::FROM_LAYER;
+    auto color = ColorP::FROM_LAYER;
     if (track.net == nullptr) {
-        c = ColorP::BUS;
+        color = ColorP::BUS;
     }
     auto width = track.width;
     if (interactive)
         object_ref_push(ObjectType::TRACK, track.uuid);
     img_net(track.net);
     img_patch_type(PatchType::TRACK);
-    img_line(track.from.get_position(), track.to.get_position(), width, track.layer);
+    if (track.is_arc()) {
+        img_arc(track.from.get_position(), track.to.get_position(), track.center.value(), track.width, track.layer);
+    }
+    else {
+        img_line(track.from.get_position(), track.to.get_position(), width, track.layer);
+    }
     img_patch_type(PatchType::OTHER);
     img_net(nullptr);
     if (interactive)
@@ -333,13 +347,20 @@ void Canvas::render(const Track &track, bool interactive)
     auto center = (track.from.get_position() + track.to.get_position()) / 2;
     if (interactive)
         object_ref_push(ObjectType::TRACK, track.uuid);
-    draw_line(track.from.get_position(), track.to.get_position(), c, layer, true, width);
+
+    if (track.is_arc()) {
+        draw_arc(track.from.get_position(), track.to.get_position(), track.center.value(), color, layer, width);
+    }
+    else {
+        draw_line(track.from.get_position(), track.to.get_position(), color, layer, true, width);
+    }
     if (track.locked) {
         auto ol = get_overlay_layer(layer);
         draw_lock(center, 0.7 * track.width, ColorP::TEXT_OVERLAY, ol, true);
     }
     if (interactive && show_text_in_tracks && width > 0 && track.net && track.net->name.size() && track.from.is_junc()
-        && track.to.is_junc() && (!show_text_in_vias || (!track.from.junc->has_via && !track.to.junc->has_via))) {
+        && track.to.is_junc() && (!show_text_in_vias || (!track.from.junc->has_via && !track.to.junc->has_via))
+        && !track.center) {
         auto overlay_layer = get_overlay_layer(track.layer, true);
         set_lod_size(width);
         const auto vec = (track.from.get_position() - track.to.get_position());
@@ -349,7 +370,7 @@ void Canvas::render(const Track &track, bool interactive)
         if (get_flip_view())
             p.invert_angle();
         p.accumulate(Placement(center));
-        p.set_angle_rad(p.get_angle_rad() + atan2(vec.y, vec.x));
+        p.set_angle_rad(p.get_angle_rad() + vec.angle());
         if (get_flip_view()) {
             p.shift.x *= -1;
             p.invert_angle();
@@ -359,9 +380,32 @@ void Canvas::render(const Track &track, bool interactive)
     }
     if (interactive)
         object_ref_pop();
-    if (interactive)
-        selectables.append_line(track.uuid, ObjectType::TRACK, track.from.get_position(), track.to.get_position(),
-                                track.width, 0, track.layer);
+    if (interactive) {
+        bool force_line = false;
+        if (track.is_arc()) {
+            Coordf a(track.from.get_position()); // ,b,c;
+            Coordf b(track.to.get_position());   // ,b,c;
+            const auto arc_center = project_onto_perp_bisector(a, b, track.center.value());
+            const float arc_radius = (arc_center - a).mag();
+            const auto arc_a0 = c2pi(atan2f(a.y - arc_center.y, a.x - arc_center.x));
+            const auto arc_a1 = c2pi(atan2f(b.y - arc_center.y, b.x - arc_center.x));
+            const float ax = std::min(asin(track.width / 2 / arc_radius),
+                                      static_cast<float>((2 * M_PI - c2pi(arc_a1 - arc_a0)) / 2 - .1e-4));
+            const float dphi = c2pi(arc_a1 - arc_a0);
+            auto mid = arc_center + Coordf::euler(arc_radius, arc_a0 + dphi / 2);
+            const auto ri = arc_radius - track.width / 2;
+            if (ri > 0)
+                selectables.append_arc_midpoint(track.uuid, ObjectType::TRACK, mid, ri, arc_radius + track.width / 2,
+                                                arc_a0 - ax, arc_a1 + ax, 0, track.layer);
+            else
+                force_line = true;
+        }
+
+        if (!track.is_arc() || force_line) {
+            selectables.append_line(track.uuid, ObjectType::TRACK, track.from.get_position(), track.to.get_position(),
+                                    track.width, 0, track.layer);
+        }
+    }
 }
 
 void Canvas::render(const BoardNetTie &tie, bool interactive)
@@ -395,7 +439,7 @@ void Canvas::render(const BoardNetTie &tie, bool interactive)
         if (get_flip_view())
             p.invert_angle();
         p.accumulate(Placement(center));
-        p.set_angle_rad(p.get_angle_rad() + atan2(vec.y, vec.x));
+        p.set_angle_rad(p.get_angle_rad() + vec.angle());
         if (get_flip_view()) {
             p.shift.x *= -1;
             p.invert_angle();
@@ -673,23 +717,28 @@ void Canvas::render(const SymbolPin &pin, SymbolMode mode, ColorP co)
 
 void Canvas::render(const Arc &arc, bool interactive, ColorP co)
 {
-    Coordf a(arc.from->position); // ,b,c;
-    Coordf b(arc.to->position);   // ,b,c;
-    Coordf c = project_onto_perp_bisector(a, b, arc.center->position);
-    const float radius0 = (c - a).mag();
-    const float a0 = c2pi(atan2f(a.y - c.y, a.x - c.x));
-    const float a1 = c2pi(atan2f(b.y - c.y, b.x - c.x));
-    const float dphi = c2pi(a1 - a0);
-    if (img_mode && img_supports_arc()) {
-        img_arc(arc);
+    if (img_mode) {
+        img_arc(arc.from->position, arc.to->position, arc.center->position, arc.width, arc.layer);
+        return;
     }
-    else {
-        draw_arc2(c, radius0, a0, a1, co, arc.layer, arc.width);
-        Coordf t(radius0, radius0);
-        if (interactive) {
+    draw_arc(arc.from->position, arc.to->position, arc.center->position, co, arc.layer, arc.width);
+    if (interactive) {
+        Coordf a(arc.from->position); // ,b,c;
+        Coordf b(arc.to->position);   // ,b,c;
+        Coordf c = project_onto_perp_bisector(a, b, arc.center->position);
+        const float radius0 = (c - a).mag();
+        const float a0 = c2pi((a - c).angle());
+        const float a1 = c2pi((b - c).angle());
+        const float dphi = c2pi(a1 - a0);
+        const auto ri = radius0 - arc.width / 2;
+
+        if (ri > 0) {
             const float ax = std::min(asin(arc.width / 2 / radius0), static_cast<float>((2 * M_PI - dphi) / 2 - .1e-4));
-            selectables.append_arc(arc.uuid, ObjectType::ARC, c, radius0 - arc.width / 2, radius0 + arc.width / 2,
-                                   a0 - ax, a1 + ax, 0, arc.layer);
+            selectables.append_arc(arc.uuid, ObjectType::ARC, c, ri, radius0 + arc.width / 2, a0 - ax, a1 + ax, 0,
+                                   arc.layer);
+        }
+        else {
+            selectables.append_line(arc.uuid, ObjectType::ARC, a, b, arc.width, 0, arc.layer);
         }
     }
 }
@@ -988,8 +1037,8 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
                 const Coordf a = v_last.position;
                 const Coordf c = project_onto_perp_bisector(a, b, v_last.arc_center);
                 const auto r = (a - c).mag();
-                float a0 = atan2f(a.y - c.y, a.x - c.x);
-                float a1 = atan2f(b.y - c.y, b.x - c.x);
+                float a0 = (a - c).angle();
+                float a1 = (b - c).angle();
                 if (v_last.arc_reverse)
                     std::swap(a0, a1);
                 selectables.append_arc(ipoly.uuid, ObjectType::POLYGON_EDGE, c, r, r, a0, a1, i_last, ipoly.layer);
@@ -1215,8 +1264,6 @@ void Canvas::render_pad_overlay(const Pad &pad, bool interactive)
         return;
     if (pad.padstack.type == Padstack::Type::MECHANICAL && !interactive)
         return;
-    transform_save();
-    transform.accumulate(pad.placement);
     auto bb = pad.padstack.get_bbox(true);  // only copper
     if (bb.second - bb.first == Coordi()) { // empty bbox
         bb = pad.padstack.get_bbox(false);  // everything
@@ -1224,6 +1271,8 @@ void Canvas::render_pad_overlay(const Pad &pad, bool interactive)
     if (bb.second - bb.first == Coordi()) { // still empty??
         return;
     }
+    transform_save();
+    transform.accumulate(pad.placement);
     auto a = bb.first;
     auto b = bb.second;
     transform.accumulate(Placement((a + b) / 2));
@@ -1353,11 +1402,21 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
         for (const auto &[pad_uuid, pad] : pkg.pads) {
             transform_save();
             transform.accumulate(pad.placement);
-            auto bb = pad.padstack.get_bbox();
+            const auto bb = pad.padstack.get_bbox();
             const auto layer = get_layer_for_padstack_type(pad.padstack.type);
             selectables.append(pad.uuid, ObjectType::PAD, {0, 0}, bb.first, bb.second, 0, layer);
             transform_restore();
             targets.emplace_back(pad.uuid, ObjectType::PAD, pad.placement.shift, 0, layer);
+            if (add_pad_bbox_targets) {
+                const auto bb_cu = pad.padstack.get_bbox(true);
+                size_t i = 1;
+                for (const auto x : {bb_cu.first.x, bb_cu.second.x}) {
+                    for (const auto y : {bb_cu.first.y, bb_cu.second.y}) {
+                        targets.emplace_back(pad.uuid, ObjectType::PAD, pad.placement.transform(Coordi(x, y)), i++,
+                                             layer);
+                    }
+                }
+            }
         }
         for (const auto &it : pkg.warnings) {
             render(it);
@@ -1521,7 +1580,7 @@ void Canvas::render(const class Dimension &dim)
         auto text_width = std::abs(text_bb.second.x - text_bb.first.x);
 
         Coordf text_pos = (q0 + v / 2) - (vn * text_width / 2);
-        auto angle = atan2(v.y, v.x);
+        auto angle = v.angle();
 
         if ((text_width + 2 * dim.label_size) > length) {
             text_pos = q1;
