@@ -14,17 +14,12 @@ private:
     const Selectables &sel;
 
     GLuint program;
+    GLuint program_arc;
     GLuint vao;
     GLuint vbo;
-
-    GLuint screenmat_loc;
-    GLuint viewmat_loc;
-    GLuint scale_loc;
-
-    GLuint color_always_loc;
-    GLuint color_inner_loc;
-    GLuint color_outer_loc;
-    GLuint color_prelight_loc;
-    GLuint min_size_loc;
+    GLuint ubo;
+    GLuint ebo;
+    unsigned int n_arc = 0;
+    unsigned int n_box = 0;
 };
 } // namespace horizon
