@@ -5,8 +5,6 @@
 #include "junction.hpp"
 #include "util/uuid_ptr.hpp"
 #include <deque>
-#include <map>
-#include <fstream>
 
 namespace horizon {
 using json = nlohmann::json;
@@ -71,6 +69,13 @@ public:
     bool is_valid() const;
     std::pair<Coordi, Coordi> get_bbox() const;
 
+    bool is_cw() const;
+    bool is_ccw() const;
+
+    void reverse();
+
+    bool is_rect() const;
+
     UUID uuid;
     std::vector<Vertex> vertices;
     int layer = 0;
