@@ -37,6 +37,9 @@ public:
         return padstack.version;
     }
 
+    void set_temp_mode();
+    void set_filename(const std::string &filename);
+
 private:
     std::map<UUID, Polygon> *get_polygon_map() override;
     std::map<UUID, Hole> *get_hole_map() override;
