@@ -268,6 +268,7 @@ std::string CoreSymbol::get_display_name(ObjectType type, const UUID &uu)
 void CoreSymbol::rebuild_internal(bool from_undo, const std::string &comment)
 {
     sym.expand(pin_display_mode);
+    sym.update_junction_connections();
     rebuild_finish(from_undo, comment);
 }
 
@@ -335,6 +336,8 @@ const std::string &CoreSymbol::get_filename() const
 
 void CoreSymbol::save(const std::string &suffix)
 {
+    if (m_filename.empty())
+        throw std::runtime_error("filename is empty");
     s_signal_save.emit();
 
     json j = sym.serialize();
@@ -348,4 +351,15 @@ void CoreSymbol::delete_autosave()
         Gio::File::create_for_path(m_filename + autosave_suffix)->remove();
 }
 
+void CoreSymbol::set_temp_mode()
+{
+    Gio::File::create_for_path(m_filename)->remove();
+    m_filename.clear();
+}
+
+void CoreSymbol::set_filename(const std::string &filename)
+{
+    m_filename = filename;
+}
+
 } // namespace horizon
