@@ -3,6 +3,7 @@
 #include "tools/tool_add_part.hpp"
 #include "tools/tool_add_vertex.hpp"
 #include "tools/tool_assign_part.hpp"
+#include "tools/tool_clear_part.hpp"
 #include "tools/tool_bend_line_net.hpp"
 #include "tools/tool_delete.hpp"
 #include "tools/tool_disconnect.hpp"
@@ -54,8 +55,6 @@
 #include "tools/tool_generate_courtyard.hpp"
 #include "tools/tool_generate_silkscreen.hpp"
 #include "tools/tool_set_group.hpp"
-#include "tools/tool_copy_placement.hpp"
-#include "tools/tool_copy_tracks.hpp"
 #include "tools/tool_swap_nets.hpp"
 #include "tools/tool_line_loop_to_polygon.hpp"
 #include "tools/tool_lines_to_tracks.hpp"
@@ -99,6 +98,8 @@
 #include "tools/tool_draw_net_tie.hpp"
 #include "tools/tool_flip_net_tie.hpp"
 #include "tools/tool_move_track_connection.hpp"
+#include "tools/tool_move_track_center.hpp"
+#include "tools/tool_paste_placement.hpp"
 
 namespace horizon {
 
@@ -208,11 +209,15 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
 
     case ToolID::PASTE:
     case ToolID::DUPLICATE:
+    case ToolID::PASTE_RELATIVE:
         return std::make_unique<ToolPaste>(this, tool_id);
 
     case ToolID::ASSIGN_PART:
         return std::make_unique<ToolAssignPart>(this, tool_id);
 
+    case ToolID::CLEAR_PART:
+        return std::make_unique<ToolClearPart>(this, tool_id);
+
     case ToolID::MAP_PACKAGE:
         return std::make_unique<ToolMapPackage>(this, tool_id);
 
@@ -322,14 +327,9 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::CLEAR_TAG:
     case ToolID::RENAME_GROUP:
     case ToolID::RENAME_TAG:
+    case ToolID::SET_TAGS_FROM_REFDES:
         return std::make_unique<ToolSetGroup>(this, tool_id);
 
-    case ToolID::COPY_PLACEMENT:
-        return std::make_unique<ToolCopyPlacement>(this, tool_id);
-
-    case ToolID::COPY_TRACKS:
-        return std::make_unique<ToolCopyTracks>(this, tool_id);
-
     case ToolID::SWAP_NETS:
         return std::make_unique<ToolSwapNets>(this, tool_id);
 
@@ -462,6 +462,12 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::MOVE_TRACK_CONNECTION:
         return std::make_unique<ToolMoveTrackConnection>(this, tool_id);
 
+    case ToolID::MOVE_TRACK_CENTER:
+        return std::make_unique<ToolMoveTrackCenter>(this, tool_id);
+
+    case ToolID::PASTE_PLACEMENT:
+        return std::make_unique<ToolPastePlacement>(this, tool_id);
+
     default:
         return nullptr;
     }
