@@ -95,9 +95,8 @@ enum class ToolID {
     SET_NEW_TAG,
     RENAME_TAG,
     CLEAR_TAG,
+    SET_TAGS_FROM_REFDES,
     TOGGLE_GROUP_TAG_VISIBLE,
-    COPY_PLACEMENT,
-    COPY_TRACKS,
     TUNE_TRACK,
     TUNE_DIFFPAIR,
     TUNE_DIFFPAIR_SKEW,
@@ -168,5 +167,9 @@ enum class ToolID {
     TIE_NETS,
     DRAW_NET_TIE,
     MOVE_TRACK_CONNECTION,
+    MOVE_TRACK_CENTER,
+    CLEAR_PART,
+    PASTE_PLACEMENT,
+    PASTE_RELATIVE,
 };
 } // namespace horizon
