@@ -24,7 +24,7 @@ bool ToolAddPart::can_begin()
 UUID ToolAddPart::create_tag()
 {
     auto uu = UUID::random();
-    auto block = doc.c->get_block();
+    auto block = doc.c->get_current_block();
     block->tag_names[uu] = std::to_string(block->tag_names.size());
     return uu;
 }
@@ -32,7 +32,7 @@ UUID ToolAddPart::create_tag()
 ToolResponse ToolAddPart::begin(const ToolArgs &args)
 {
     std::cout << "tool add part\n";
-    Schematic *sch = doc.c->get_schematic();
+    Schematic *sch = doc.c->get_current_schematic();
 
     if (tool_id == ToolID::ADD_PART) {
         UUID part_uuid;
@@ -65,7 +65,7 @@ ToolResponse ToolAddPart::begin(const ToolArgs &args)
             return ToolResponse::end();
         }
     }
-    comp->refdes = comp->entity->prefix + "?";
+    comp->refdes = comp->get_prefix() + "?";
     comp->tag = create_tag();
 
     for (auto &it : comp->entity->gates) {
@@ -111,26 +111,26 @@ ToolResponse ToolAddPart::update(const ToolArgs &args)
     else if (args.type == ToolEventType::ACTION) {
         switch (args.action) {
         case InToolActionID::LMB:
-            doc.c->get_schematic()->autoconnect_symbol(doc.c->get_sheet(), sym_current);
+            doc.c->get_current_schematic()->autoconnect_symbol(doc.c->get_sheet(), sym_current);
             if (sym_current->component->connections.size() == 0) {
-                doc.c->get_schematic()->place_bipole_on_line(doc.c->get_sheet(), sym_current);
+                doc.c->get_current_schematic()->place_bipole_on_line(doc.c->get_sheet(), sym_current);
             }
             if (current_gate + 1 == gates.size()) { // last gate
                 current_gate = 0;
                 // add next component
                 auto last_comp = comp;
                 auto uu = UUID::random();
-                Schematic *sch = doc.c->get_schematic();
+                Schematic *sch = doc.c->get_current_schematic();
                 comp = &sch->block->components.emplace(uu, uu).first->second;
                 comp->entity = last_comp->entity;
                 comp->part = last_comp->part;
-                comp->refdes = comp->entity->prefix + "?";
+                comp->refdes = comp->get_prefix() + "?";
                 comp->tag = create_tag();
 
                 auto old_symbol = sym_current;
                 sym_current = map_symbol(comp, gates.front());
                 if (!sym_current) {
-                    doc.c->get_block()->components.erase(comp->uuid);
+                    doc.c->get_current_block()->components.erase(comp->uuid);
                     return ToolResponse::commit();
                 }
                 sym_current->placement = old_symbol->placement;
@@ -160,9 +160,9 @@ ToolResponse ToolAddPart::update(const ToolArgs &args)
         case InToolActionID::RMB:
         case InToolActionID::CANCEL:
             if (current_gate == 0) { // also delete the component
-                doc.c->get_block()->components.erase(comp->uuid);
+                doc.c->get_current_block()->components.erase(comp->uuid);
             }
-            doc.c->delete_schematic_symbol(sym_current->uuid);
+            doc.c->get_sheet()->symbols.erase(sym_current->uuid);
             sym_current = nullptr;
             return ToolResponse::commit();
             break;
