@@ -4,8 +4,9 @@
 #include "board/board.hpp"
 #include "imp/imp_interface.hpp"
 #include "pool/part.hpp"
-#include <iostream>
 #include "core/tool_id.hpp"
+#include "core/tool_data_window.hpp"
+#include "dialogs/edit_plane_window.hpp"
 
 namespace horizon {
 
@@ -54,19 +55,19 @@ bool ToolEditPlane::can_begin()
 
 ToolResponse ToolEditPlane::begin(const ToolArgs &args)
 {
-    std::cout << "tool edit plane\n";
-
     auto poly = get_poly();
     Plane *plane = nullptr;
     auto brd = doc.b->get_board();
-    std::set<UUID> nets;
     if (tool_id == ToolID::EDIT_PLANE || tool_id == ToolID::UPDATE_PLANE || tool_id == ToolID::CLEAR_PLANE) {
         plane = dynamic_cast<Plane *>(poly->usage.ptr);
-        nets.insert(plane->net->uuid);
         if (tool_id == ToolID::UPDATE_PLANE) {
-            brd->update_plane(plane);
-            brd->update_airwires(false, {plane->net->uuid});
-            return ToolResponse::commit();
+            if (imp->dialogs.update_plane(*brd, plane)) {
+                brd->update_airwires(false, {plane->net->uuid});
+                return ToolResponse::commit();
+            }
+            else {
+                return ToolResponse::revert();
+            }
         }
         else if (tool_id == ToolID::CLEAR_PLANE) {
             plane->clear();
@@ -82,30 +83,11 @@ ToolResponse ToolEditPlane::begin(const ToolArgs &args)
         plane->polygon = poly;
         poly->usage = plane;
     }
-    UUID plane_uuid = plane->uuid;
-    bool r = imp->dialogs.edit_plane(*plane, *brd);
-    if (r) {
-        if (brd->planes.count(plane_uuid)) { // may have been deleted
-            nets.insert(plane->net->uuid);
-            if (tool_id == ToolID::ADD_PLANE && plane->from_rules) {
-                plane->settings = dynamic_cast<const BoardRules &>(*doc.b->get_rules())
-                                          .get_plane_settings(plane->net, poly->layer);
-            }
-            brd->update_plane(plane);
-            brd->update_airwires(false, nets);
-        }
-        else {
-            poly->usage = nullptr;
-            brd->update_planes();
-        }
-        return ToolResponse::commit();
-    }
-    else {
-        return ToolResponse::revert();
-    }
+    show_edit_plane_window(*plane, *brd);
+    return ToolResponse();
 }
 ToolResponse ToolEditPlane::update(const ToolArgs &args)
 {
-    return ToolResponse();
+    return update_for_plane(args);
 }
 } // namespace horizon
