@@ -79,6 +79,25 @@ void ToolMove::expand_selection()
         case ObjectType::POWER_SYMBOL: {
             auto &ps = doc.c->get_sheet()->power_symbols.at(it.uuid);
             new_sel.emplace(ps.junction->uuid, ObjectType::JUNCTION);
+            for (const auto &uu_line : ps.junction->connected_net_lines) {
+                const auto &line = doc.c->get_sheet()->net_lines.at(uu_line);
+                for (const auto &it_ft : {line.from, line.to}) {
+                    if (it_ft.is_junc()) {
+                        const auto &ju = *it_ft.junc;
+                        const auto pin_pos = ps.junction->position;
+                        Axis ax = Axis::NONE;
+                        if (pin_pos.x == ju.position.x)
+                            ax = Axis::X;
+                        else if (pin_pos.y == ju.position.y)
+                            ax = Axis::Y;
+
+                        if (ju.connected_net_lines.size() == 1) { // dangling end
+                            ax = Axis::X | Axis::Y;
+                        }
+                        add_extra_junction(ju.uuid, ax);
+                    }
+                }
+            }
         } break;
         case ObjectType::BUS_RIPPER: {
             auto &rip = doc.c->get_sheet()->bus_rippers.at(it.uuid);
@@ -93,6 +112,16 @@ void ToolMove::expand_selection()
                 }
             }
         } break;
+        case ObjectType::SCHEMATIC_NET_TIE: {
+            auto &tie = doc.c->get_sheet()->net_ties.at(it.uuid);
+            new_sel.emplace(tie.from->uuid, ObjectType::JUNCTION);
+            new_sel.emplace(tie.to->uuid, ObjectType::JUNCTION);
+        } break;
+        case ObjectType::BOARD_NET_TIE: {
+            auto &tie = doc.b->get_board()->net_ties.at(it.uuid);
+            new_sel.emplace(tie.from->uuid, ObjectType::JUNCTION);
+            new_sel.emplace(tie.to->uuid, ObjectType::JUNCTION);
+        } break;
         case ObjectType::TRACK: {
             auto track = &doc.b->get_board()->tracks.at(it.uuid);
             for (auto &it_ft : {track->from, track->to}) {
@@ -310,14 +339,18 @@ ToolResponse ToolMove::begin(const ToolArgs &args)
             move_mirror_or_rotate(selection_center, true);
             move_mirror_or_rotate(selection_center, true);
         }
-        finish();
-        return ToolResponse::commit();
+        if (finish())
+            return ToolResponse::commit();
+        else
+            return ToolResponse::revert();
     }
     if (tool_id == ToolID::MOVE_EXACTLY) {
         if (auto r = imp->dialogs.ask_datum_coord("Move exactly")) {
             move_do(*r);
-            finish();
-            return ToolResponse::commit();
+            if (finish())
+                return ToolResponse::commit();
+            else
+                return ToolResponse::revert();
         }
         else {
             return ToolResponse::end();
@@ -443,16 +476,22 @@ void ToolMove::do_move(const Coordi &d)
     update_tip();
 }
 
-void ToolMove::finish()
+bool ToolMove::finish()
 {
-    merge_and_connect();
+    std::set<UUID> junctions;
+    for (const auto &[uu, ax] : extra_junctions)
+        junctions.emplace(uu);
+    merge_and_connect(junctions);
     if (doc.b) {
+        imp->tool_bar_set_tip("updating planes…");
         auto brd = doc.b->get_board();
         for (auto plane : planes) {
-            brd->update_plane(plane);
+            if (!imp->dialogs.update_plane(*brd, plane))
+                return false;
         }
         brd->update_airwires(false, nets);
     }
+    return true;
 }
 
 ToolResponse ToolMove::update(const ToolArgs &args)
@@ -465,8 +504,10 @@ ToolResponse ToolMove::update(const ToolArgs &args)
     else if (args.type == ToolEventType::ACTION) {
         if (any_of(args.action, {InToolActionID::LMB, InToolActionID::COMMIT})
             || (is_transient && args.action == InToolActionID::LMB_RELEASE)) {
-            finish();
-            return ToolResponse::commit();
+            if (finish())
+                return ToolResponse::commit();
+            else
+                return ToolResponse::revert();
         }
         else if (any_of(args.action, {InToolActionID::RMB, InToolActionID::CANCEL})) {
             return ToolResponse::revert();
