@@ -11,15 +11,22 @@
 #include "pool/entity.hpp"
 #include "util/util.hpp"
 #include "util/geom_util.hpp"
+#include "util/selection_util.hpp"
 #include "core/clipboard/clipboard.hpp"
 #include <iostream>
 #include <gtkmm.h>
 #include "core/tool_id.hpp"
 #include "util/picture_util.hpp"
 #include "blocks/blocks_schematic.hpp"
+#include "pool/pool.hpp"
+#include <filesystem>
+#include "tool_data_pool_updated.hpp"
+#include "util/pasted_package.hpp"
+#include "util/range_util.hpp"
+#include "board/board_layers.hpp"
 
 namespace horizon {
-
+namespace fs = std::filesystem;
 ToolPaste::ToolPaste(IDocument *c, ToolID tid) : ToolBase(c, tid), ToolHelperMove(c, tid), ToolHelperMerge(c, tid)
 {
 }
@@ -28,13 +35,15 @@ bool ToolPaste::can_begin()
 {
     if (tool_id == ToolID::PASTE)
         return true;
+    else if (tool_id == ToolID::PASTE_RELATIVE)
+        return doc.b && (sel_count_type(selection, ObjectType::BOARD_PACKAGE) == 1);
     else
         return selection.size();
 }
 
 bool ToolPaste::is_specific()
 {
-    return tool_id == ToolID::DUPLICATE;
+    return tool_id != ToolID::PASTE;
 }
 
 class JunctionProvider : public ObjectProvider {
@@ -61,11 +70,34 @@ void ToolPaste::fix_layer(int &la)
     if (doc.r->get_layer_provider().get_layers().count(la) == 0) {
         la = 0;
     }
+    if (doc.b && ref_pkg && target_pkg && ref_pkg->flip != target_pkg->flip) {
+        doc.b->get_board()->flip_package_layer(la);
+    }
 }
 
-void ToolPaste::apply_shift(Coordi &c, const Coordi &cursor_pos)
+void ToolPaste::transform(Coordi &c) const
 {
-    c += shift;
+    Placement p{c};
+    transform(p, ObjectType::BOARD_PACKAGE);
+    c = p.shift;
+}
+
+void ToolPaste::transform(Placement &p, ObjectType type) const
+{
+    if (target_pkg && ref_pkg) {
+        switch (type) {
+        case ObjectType::BOARD_PACKAGE:
+        case ObjectType::BOARD_DECAL:
+            p = transform_package_placement_to_new_reference(p, ref_pkg->placement, target_pkg->placement);
+            break;
+
+        default:
+            p = transform_text_placement_to_new_reference(p, ref_pkg->placement, target_pkg->placement);
+        }
+    }
+    else {
+        p.shift += shift;
+    }
 }
 
 class ToolDataPaste : public ToolData {
@@ -81,22 +113,128 @@ public:
 };
 
 ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canvas)
+{
+    ItemSet items_needed;
+    if (doc.c && j.count("pool_items_needed") && j.count("pool_base_path")
+        && j.at("pool_base_path").get<std::string>() != doc.r->get_pool().get_base_path()) {
+        auto &db = doc.r->get_pool().get_db();
+        db.execute("CREATE TEMP TABLE 'paste_items_needed' ('type' TEXT NOT NULL, 'uuid' TEXT NOT NULL);");
+        {
+            SQLite::Query q_insert(db, "INSERT INTO paste_items_NEEDED VALUES (?, ?)");
+            for (const auto &it : j.at("pool_items_needed")) {
+                q_insert.reset();
+                q_insert.bind(1, it.at(0).get<std::string>());
+                q_insert.bind(2, it.at(1).get<std::string>());
+                q_insert.step();
+            }
+        }
+        SQLite::Query q(db,
+                        "SELECT paste_items_needed.type, paste_items_needed.uuid FROM paste_items_needed LEFT JOIN "
+                        "all_items_view USING (uuid) WHERE "
+                        "all_items_view.uuid IS NULL");
+        while (q.step()) {
+            items_needed.emplace(object_type_lut.lookup(q.get<std::string>(0)), q.get<std::string>(1));
+        }
+        db.execute("DROP TABLE paste_items_needed");
+    }
+    if (items_needed.size()) {
+        Pool paste_pool(j.at("pool_base_path").get<std::string>());
+        for (const auto &[type, uu] : items_needed) {
+            const auto src_filename = fs::u8path(paste_pool.get_filename(type, uu));
+            const auto dest_dir = fs::u8path(doc.r->get_pool().get_base_path()) / Pool::type_names.at(type) / "cache";
+            fs::create_directories(dest_dir);
+            if (type == ObjectType::PACKAGE) {
+                const auto dest_pkg_dir = dest_dir / (((std::string)uu));
+                fs::create_directories(dest_pkg_dir);
+                fs::copy(src_filename, dest_pkg_dir / "package.json");
+            }
+            else if (type == ObjectType::PADSTACK) {
+                SQLite::Query q(paste_pool.db, "SELECT package FROM padstacks WHERE uuid = ?");
+                q.bind(1, uu);
+                if (q.step()) {
+                    const UUID package_uuid = q.get<std::string>(0);
+                    if (package_uuid) {
+                        const auto dest_padstack_dir = fs::u8path(doc.r->get_pool().get_base_path())
+                                                       / Pool::type_names.at(ObjectType::PACKAGE) / "cache"
+                                                       / (std::string)package_uuid / "padstacks";
+                        fs::create_directories(dest_padstack_dir);
+                        fs::copy(src_filename, dest_padstack_dir / (((std::string)uu) + ".json"));
+                    }
+                    else {
+                        fs::copy(src_filename, dest_dir / (((std::string)uu) + ".json"));
+                    }
+                }
+                else {
+                    throw std::runtime_error("padstack not found???");
+                }
+            }
+            else {
+                fs::copy(src_filename, dest_dir / (((std::string)uu) + ".json"));
+            }
+        }
+        imp->pool_update({});
+        pool_update_pending = true;
+        update_tip();
+        imp->tool_bar_set_actions({
+                {InToolActionID::RMB},
+        });
+        return ToolResponse();
+    }
+    else {
+        return really_begin_paste(j, cursor_pos_canvas);
+    }
+}
+
+ToolResponse ToolPaste::really_begin_paste(const json &j, const Coordi &cursor_pos_canvas)
 {
     Coordi cursor_pos = j.at("cursor_pos").get<std::vector<int64_t>>();
-    printf("Core %p\n", doc.r);
     selection.clear();
     shift = cursor_pos_canvas - cursor_pos;
+    std::map<UUID, PastedPackage> pkgs_from_paste;
+    std::map<UUID, Net *> board_net_xlat;
+    std::set<UUID> pkg_texts;
+    if (target_pkg) {
+        for (const auto &[k, v] : j.at("packages").items()) {
+            pkgs_from_paste.emplace(std::piecewise_construct, std::forward_as_tuple(k), std::forward_as_tuple(k, v));
+        }
+        // find target pkg in pkgs_from_paste
+        auto pkgs_from_paste_values = pkgs_from_paste | ranges::views::values;
+        ref_pkg = find_if_ptr(pkgs_from_paste_values, [this](auto &x) { return x.tag == target_pkg->component->tag; });
+        if (!ref_pkg) {
+            imp->tool_bar_flash("reference package not found");
+            return ToolResponse::end();
+        }
+        auto pkgs_from_board = doc.b->get_board()->packages | ranges::views::values;
+        for (const auto &ppkg : pkgs_from_paste_values) {
+            pkg_texts.insert(ppkg.texts.begin(), ppkg.texts.end());
+            auto bpkg = find_if_ptr(pkgs_from_board, [this, &ppkg](const auto &x) {
+                return x.component->tag == ppkg.tag && x.component->group == ref_pkg->group;
+            });
+            if (bpkg && bpkg->package.uuid == ppkg.package) {
+                auto conns =
+                        ppkg.connections
+                        | ranges::views::filter([bpkg](const auto &x) { return bpkg->package.pads.count(x.first); })
+                        | ranges::views::transform([bpkg](const auto &x) {
+                              return std::make_pair(x.second, bpkg->package.pads.at(x.first).net);
+                          })
+                        | ranges::views::filter([](const auto &x) { return x.second; });
+                board_net_xlat.insert(conns.begin(), conns.end());
+            }
+        }
+    }
 
     std::map<UUID, const UUID> text_xlat;
     if (j.count("texts") && doc.r->has_object_type(ObjectType::TEXT)) {
         const json &o = j["texts"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            if (pkg_texts.count(it.key()))
+                continue;
             auto u = UUID::random();
             text_xlat.emplace(it.key(), u);
             auto x = doc.r->insert_text(u);
             *x = Text(u, it.value());
+            transform(x->placement, ObjectType::TEXT);
             fix_layer(x->layer);
-            apply_shift(x->placement.shift, cursor_pos_canvas);
             selection.emplace(u, ObjectType::TEXT);
         }
     }
@@ -106,8 +244,8 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             auto u = UUID::random();
             auto x = doc.r->insert_dimension(u);
             *x = Dimension(u, it.value());
-            apply_shift(x->p0, cursor_pos_canvas);
-            apply_shift(x->p1, cursor_pos_canvas);
+            transform(x->p0);
+            transform(x->p1);
             selection.emplace(u, ObjectType::DIMENSION, 0);
             selection.emplace(u, ObjectType::DIMENSION, 1);
         }
@@ -128,11 +266,11 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                 if (block.nets.count(net_uu))
                     junction_nets.emplace(u, &block.nets.at(net_uu));
             }
-            apply_shift(x->position, cursor_pos_canvas);
+            transform(x->position);
             selection.emplace(u, ObjectType::JUNCTION);
         }
     }
-    if (j.count("lines")) {
+    if (j.count("lines") && doc.r->has_object_type(ObjectType::LINE)) {
         const json &o = j["lines"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
@@ -143,7 +281,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             selection.emplace(u, ObjectType::LINE);
         }
     }
-    if (j.count("arcs")) {
+    if (j.count("arcs") && doc.r->has_object_type(ObjectType::ARC)) {
         const json &o = j["arcs"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
@@ -161,7 +299,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             auto &x = doc.k->get_package().pads.emplace(u, Pad(u, it.value(), doc.r->get_pool())).first->second;
-            apply_shift(x.placement.shift, cursor_pos_canvas);
+            transform(x.placement, ObjectType::PAD);
             pads.push_back(&x);
             selection.emplace(u, ObjectType::PAD);
         }
@@ -175,17 +313,18 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             }
         }
     }
-    if (j.count("holes")) {
+    if (j.count("holes") && doc.a) {
         const json &o = j["holes"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             auto x = doc.r->insert_hole(u);
             *x = Hole(u, it.value());
-            apply_shift(x->placement.shift, cursor_pos_canvas);
+            transform(x->placement, ObjectType::HOLE);
             selection.emplace(u, ObjectType::HOLE);
         }
     }
-    if (j.count("polygons")) {
+    std::map<UUID, Polygon *> polygon_xlat;
+    if (j.count("polygons") && doc.r->has_object_type(ObjectType::POLYGON)) {
         const json &o = j["polygons"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
@@ -193,15 +332,29 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             *x = Polygon(u, it.value());
             int vertex = 0;
             for (auto &itv : x->vertices) {
-                itv.arc_center += shift;
-                itv.position += shift;
+                transform(itv.arc_center);
+                transform(itv.position);
                 if (itv.type == Polygon::Vertex::Type::ARC)
                     selection.emplace(u, ObjectType::POLYGON_ARC_CENTER, vertex);
                 selection.emplace(u, ObjectType::POLYGON_VERTEX, vertex++);
             }
+            polygon_xlat.emplace(it.key(), x);
+            fix_layer(x->layer);
+        }
+    }
+    if (j.count("planes") && doc.b) {
+        for (const auto &[uu, it] : j.at("planes").items()) {
+            Plane plane{UUID::random(), it, nullptr};
+            if (polygon_xlat.count(plane.polygon.uuid) && board_net_xlat.count(plane.net.uuid)) {
+                plane.polygon = polygon_xlat.at(plane.polygon.uuid);
+                plane.net = board_net_xlat.at(plane.net.uuid);
+                auto &x = doc.b->get_board()->planes.emplace(plane.uuid, plane).first->second;
+                x.polygon->usage = &x;
+            }
         }
     }
     std::map<UUID, const UUID> net_xlat;
+    std::set<Net *> new_nets;
     if (j.count("nets") && doc.c && doc.c->get_current_block()) {
         const json &o = j["nets"];
         auto block = doc.c->get_current_block();
@@ -223,10 +376,24 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                 net_new->is_power = net_from_json.is_power;
                 net_new->name = net_name;
                 net_new->power_symbol_style = net_from_json.power_symbol_style;
+                if (net_from_json.diffpair_primary) {
+                    net_new->diffpair_primary = true;
+                    net_new->diffpair.uuid = net_from_json.diffpair.uuid;
+                }
+                new_nets.insert(net_new);
             }
             if (net_new)
                 net_xlat.emplace(it.key(), net_new->uuid);
         }
+        for (auto net : new_nets) {
+            if (net->diffpair_primary) {
+                if (net_xlat.count(net->diffpair.uuid))
+                    net->diffpair = &block->nets.at(net_xlat.at(net->diffpair.uuid));
+                else
+                    net->diffpair_primary = false;
+            }
+        }
+        block->update_diffpairs();
     }
 
     std::map<UUID, const UUID> component_xlat;
@@ -242,6 +409,13 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                     it_conn.second.net = &block->nets.at(net_xlat.at(it_conn.second.net.uuid));
             }
             block->components.emplace(u, comp);
+            if (comp.group && !block->group_names.count(comp.group)) {
+                block->group_names.emplace(comp.group,
+                                           j.at("group_names").at((std::string)comp.group).get<std::string>());
+            }
+            if (comp.tag && !block->tag_names.count(comp.tag)) {
+                block->tag_names.emplace(comp.tag, j.at("tag_names").at((std::string)comp.tag).get<std::string>());
+            }
         }
     }
     std::map<UUID, const UUID> symbol_xlat;
@@ -260,7 +434,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                 it_txt = doc.r->get_text(text_xlat.at(it_txt.uuid));
             }
             sheet->expand_symbol(u, *doc.c->get_current_schematic());
-            x->placement.shift += shift;
+            transform(x->placement, ObjectType::SCHEMATIC_SYMBOL);
             selection.emplace(u, ObjectType::SCHEMATIC_SYMBOL);
         }
     }
@@ -304,7 +478,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                 auto &x = sheet->block_symbols.emplace(u, sym).first->second;
                 sheet->expand_block_symbol(u, *doc.c->get_current_schematic());
                 doc.c->get_top_schematic()->update_sheet_mapping();
-                x.placement.shift += shift;
+                transform(x.placement, ObjectType::SCHEMATIC_BLOCK_SYMBOL);
                 selection.emplace(u, ObjectType::SCHEMATIC_BLOCK_SYMBOL);
             }
         }
@@ -416,7 +590,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             auto &x = doc.a->get_padstack().shapes.emplace(u, Shape(u, it.value())).first->second;
-            x.placement.shift += shift;
+            transform(x.placement, ObjectType::SHAPE);
             selection.emplace(u, ObjectType::SHAPE);
         }
     }
@@ -432,11 +606,15 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             x->expand(*brd);
             if (brd->block->nets.count(x->net_set.uuid)) {
                 x->net_set = &brd->block->nets.at(x->net_set.uuid);
-                nets.insert(x->net_set->uuid);
+            }
+            else if (board_net_xlat.count(x->net_set.uuid)) {
+                x->net_set = board_net_xlat.at(x->net_set.uuid);
             }
             else {
                 x->net_set = nullptr;
             }
+            if (x->net_set)
+                nets.insert(x->net_set->uuid);
             x->junction = &brd->junctions.at(junction_xlat.at(x->junction.uuid));
             x->junction->net = x->net_set;
             selection.emplace(u, ObjectType::VIA);
@@ -453,6 +631,34 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                               .first->second;
             x->to.junc = &brd->junctions.at(junction_xlat.at(x->to.junc.uuid));
             x->from.junc = &brd->junctions.at(junction_xlat.at(x->from.junc.uuid));
+            fix_layer(x->layer);
+            struct ConnInfo {
+                ConnInfo(const std::string &w, Track::Connection &c) : where(w), conn(c)
+                {
+                }
+                const std::string where;
+                Track::Connection &conn;
+            };
+            std::vector<ConnInfo> conns = {{"from_pad", x->from}, {"to_pad", x->to}};
+            auto board_pkgs = brd->packages | ranges::views::values;
+            for (auto &conn_info : conns) {
+                if (it.value().count(conn_info.where)) {
+                    Track::Connection conn{it.value().at(conn_info.where)};
+                    if (pkgs_from_paste.count(conn.package.uuid)) {
+                        const auto &ppkg = pkgs_from_paste.at(conn.package.uuid);
+                        auto bpkg = find_if_ptr(board_pkgs, [this, &ppkg](auto &k) {
+                            return k.component->tag == ppkg.tag && k.component->group == target_pkg->component->group;
+                        });
+                        if (bpkg && bpkg->package.pads.count(conn.pad.uuid)) {
+                            auto &pad = bpkg->package.pads.at(conn.pad.uuid);
+                            conn_info.conn.connect(bpkg, &pad);
+                            if (pad.net)
+                                doc.b->get_board()->airwires_expand.insert(pad.net->uuid);
+                            doc.b->get_board()->expand_flags |= Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_AIRWIRES;
+                        }
+                    }
+                }
+            }
             selection.emplace(u, ObjectType::TRACK);
         }
     }
@@ -472,7 +678,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                 x->net = nullptr;
             }
             x->padstack.apply_parameter_set(x->parameter_set);
-            apply_shift(x->placement.shift, cursor_pos_canvas);
+            transform(x->placement, ObjectType::BOARD_HOLE);
             selection.emplace(u, ObjectType::BOARD_HOLE);
         }
     }
@@ -488,7 +694,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                                   .emplace(std::piecewise_construct, std::forward_as_tuple(u),
                                            std::forward_as_tuple(u, it.value(), *brd))
                                   .first->second;
-                apply_shift(x->placement.shift, cursor_pos_canvas);
+                transform(x->placement, ObjectType::BOARD_PANEL);
                 selection.emplace(u, ObjectType::BOARD_PANEL);
             }
         }
@@ -502,7 +708,8 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                               .emplace(std::piecewise_construct, std::forward_as_tuple(u),
                                        std::forward_as_tuple(u, it.value(), doc.r->get_pool_caching(), *brd))
                               .first->second;
-            apply_shift(x.placement.shift, cursor_pos_canvas);
+            transform(x.placement, ObjectType::BOARD_DECAL);
+            x.set_flip(x.placement.mirror, *brd);
             selection.emplace(u, ObjectType::BOARD_DECAL);
         }
     }
@@ -531,12 +738,8 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         }
     }
     update_airwires();
-    return ToolResponse();
-}
-
-ToolResponse ToolPaste::begin(const ToolArgs &args)
-{
-    update_tip();
+    if (tool_id == ToolID::PASTE_RELATIVE)
+        return ToolResponse::commit();
     imp->tool_bar_set_actions({
             {InToolActionID::LMB},
             {InToolActionID::RMB},
@@ -544,11 +747,21 @@ ToolResponse ToolPaste::begin(const ToolArgs &args)
             {InToolActionID::MIRROR},
             {InToolActionID::RESTRICT},
     });
+    return ToolResponse();
+}
+
+ToolResponse ToolPaste::begin(const ToolArgs &args)
+{
+    update_tip();
     if (auto data = dynamic_cast<ToolDataPaste *>(args.data.get())) {
         paste_data = data->paste_data;
         return begin_paste(paste_data, args.coords);
     }
 
+    if (tool_id == ToolID::PASTE_RELATIVE) {
+        target_pkg = &doc.b->get_board()->packages.at(sel_find_one(selection, ObjectType::BOARD_PACKAGE).uuid);
+    }
+
     if (tool_id == ToolID::DUPLICATE) {
         auto clip = ClipboardBase::create(*doc.r);
         paste_data = clip->process(args.selection);
@@ -581,6 +794,9 @@ void ToolPaste::update_tip()
     if (paste_data == nullptr && pic == nullptr) { // wait for data
         imp->tool_bar_set_tip("waiting for paste data");
     }
+    else if (pool_update_pending) {
+        imp->tool_bar_set_tip("waiting for pool update");
+    }
     else {
         auto delta = get_delta();
         imp->tool_bar_set_tip(coord_to_string(delta, true) + " " + restrict_mode_to_string());
@@ -591,29 +807,40 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
 {
     if (paste_data == nullptr && pic == nullptr) { // wait for data
         if (args.type == ToolEventType::DATA) {
-            auto data = dynamic_cast<ToolDataPaste *>(args.data.get());
-            if (data->paste_data != nullptr) {
-                paste_data = data->paste_data;
-                return begin_paste(paste_data, args.coords);
-            }
-            else if (data->pixbuf) {
-                pic = doc.r->insert_picture(UUID::random());
-                pic->placement.shift = args.coords;
-                pic->data = picture_data_from_pixbuf(data->pixbuf);
-                pic->data_uuid = pic->data->uuid;
-                float width = 10_mm;
-                pic->px_size = width / pic->data->width;
-                selection.clear();
-                selection.emplace(pic->uuid, ObjectType::PICTURE);
-                update_tip();
-                move_init(args.coords);
-            }
-            else {
-                imp->tool_bar_flash("Empty Buffer");
-                return ToolResponse::end();
+            if (auto data = dynamic_cast<ToolDataPaste *>(args.data.get())) {
+                if (data->paste_data != nullptr) {
+                    paste_data = data->paste_data;
+                    return begin_paste(paste_data, args.coords);
+                }
+                else if (data->pixbuf) {
+                    pic = doc.r->insert_picture(UUID::random());
+                    pic->placement.shift = args.coords;
+                    pic->data = picture_data_from_pixbuf(data->pixbuf);
+                    pic->data_uuid = pic->data->uuid;
+                    float width = 10_mm;
+                    pic->px_size = width / pic->data->width;
+                    selection.clear();
+                    selection.emplace(pic->uuid, ObjectType::PICTURE);
+                    update_tip();
+                    move_init(args.coords);
+                }
+                else {
+                    imp->tool_bar_flash("Empty Buffer");
+                    return ToolResponse::end();
+                }
             }
         }
     }
+    else if (pool_update_pending) {
+        if (dynamic_cast<ToolDataPoolUpdated *>(args.data.get())) {
+            pool_update_pending = false;
+            return really_begin_paste(paste_data, args.coords);
+        }
+        else if (args.type == ToolEventType::ACTION
+                 && any_of(args.action, {InToolActionID::RMB, InToolActionID::CANCEL})) {
+            return ToolResponse::end();
+        }
+    }
     else {
         if (args.type == ToolEventType::MOVE) {
             move_do_cursor(args.coords);
