@@ -1,23 +1,17 @@
 #include "tool_place_junction.hpp"
-#include "document/idocument_schematic.hpp"
-#include "schematic/schematic.hpp"
-#include "document/idocument_symbol.hpp"
-#include "pool/symbol.hpp"
+#include "document/idocument.hpp"
 #include "imp/imp_interface.hpp"
+#include "common/junction.hpp"
 #include <iostream>
 
 namespace horizon {
 
-ToolPlaceJunction::ToolPlaceJunction(IDocument *c, ToolID tid) : ToolBase(c, tid)
-{
-}
-
-bool ToolPlaceJunction::can_begin()
+bool ToolPlaceJunctionBase::can_begin()
 {
     return doc.r->has_object_type(ObjectType::JUNCTION);
 }
 
-ToolResponse ToolPlaceJunction::begin(const ToolArgs &args)
+ToolResponse ToolPlaceJunctionBase::begin(const ToolArgs &args)
 {
     std::cout << "tool place junction\n";
 
@@ -36,39 +30,35 @@ ToolResponse ToolPlaceJunction::begin(const ToolArgs &args)
     return ToolResponse();
 }
 
-void ToolPlaceJunction::create_junction(const Coordi &c)
+void ToolPlaceJunction::insert_junction()
 {
     temp = doc.r->insert_junction(UUID::random());
-    imp->set_snap_filter({{ObjectType::JUNCTION, temp->uuid}});
-    temp->position = c;
 }
 
-ToolResponse ToolPlaceJunction::update(const ToolArgs &args)
+void ToolPlaceJunctionBase::create_junction(const Coordi &c)
+{
+    insert_junction();
+    imp->set_snap_filter({{ObjectType::JUNCTION, get_junction()->uuid}});
+    get_junction()->position = c;
+}
+
+ToolResponse ToolPlaceJunctionBase::update(const ToolArgs &args)
 {
     if (update_attached(args)) {
         return ToolResponse();
     }
 
     if (args.type == ToolEventType::MOVE) {
-        temp->position = args.coords;
+        get_junction()->position = args.coords;
         return ToolResponse();
     }
     else if (args.type == ToolEventType::ACTION) {
         switch (args.action) {
         case InToolActionID::LMB:
-            if (doc.c) {
-                for (auto it : doc.c->get_net_lines()) {
-                    if (it->coord_on_line(temp->position)) {
-                        std::cout << "on line" << std::endl;
-                        if (!check_line(it))
-                            return ToolResponse();
-
-                        doc.c->get_sheet()->split_line_net(it, temp);
-                        break;
-                    }
-                }
+            if (junction_placed()) {
+                return ToolResponse();
             }
-            junctions_placed.push_front(temp);
+            junctions_placed.push_front(get_junction());
 
             create_junction(args.coords);
             create_attached();
@@ -77,12 +67,12 @@ ToolResponse ToolPlaceJunction::update(const ToolArgs &args)
         case InToolActionID::RMB:
         case InToolActionID::CANCEL:
             delete_attached();
-            doc.r->delete_junction(temp->uuid);
-            temp = 0;
+            doc.r->delete_junction(get_junction()->uuid);
             selection.clear();
             for (auto it : junctions_placed) {
                 selection.emplace(it->uuid, ObjectType::JUNCTION);
             }
+            finish();
             return ToolResponse::commit();
 
         default:;
