@@ -1,11 +1,11 @@
 #pragma once
 #include "core/tool.hpp"
-#include "package/pad.hpp"
+#include "parameter/set.hpp"
 
 namespace horizon {
 class ToolPlacePad : public ToolBase {
 public:
-    ToolPlacePad(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
@@ -18,8 +18,9 @@ public:
     }
 
 private:
-    const Padstack *padstack = nullptr;
-    Pad *temp = nullptr;
+    const class Padstack *padstack = nullptr;
+    class Pad *temp = nullptr;
+    ParameterSet temp_param;
     void create_pad(const Coordi &c);
 };
 } // namespace horizon
