@@ -5,7 +5,7 @@ namespace horizon {
 
 class ToolRenumberPads : public ToolBase {
 public:
-    ToolRenumberPads(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
