@@ -36,6 +36,7 @@ public:
     std::optional<UUID> select_included_board(const class Board &brd);
     std::optional<UUID> select_decal(class IPool &pool);
     std::optional<UUID> select_block(const class BlocksSchematic &blocks);
+    std::optional<UUID> map_net_tie(const std::set<class NetTie *> &net_ties);
 
     unsigned int ask_net_merge(class Net &net, class Net &into);
     bool manage_buses(class Block &b);
@@ -45,16 +46,15 @@ public:
     bool edit_board_hole(std::set<class BoardHole *> &holes, class IPool &pool, class IPool &pool_caching,
                          class Block &block);
     bool annotate(class Schematic &s);
-    bool edit_plane(class Plane &plane, class Board &brd);
     bool edit_keepout(class Keepout &keepout, class IDocument &c, bool add_mode);
     bool edit_stackup(class IDocumentBoard &brd);
-    bool edit_schematic_properties(class IDocumentSchematicBlockSymbol &s);
+    bool edit_schematic_properties(class IDocumentSchematicBlockSymbol &s, const UUID &block, const UUID &sheet);
+    bool edit_project_properties(class Block &b);
     bool edit_frame_properties(class Frame &fr);
     std::optional<int64_t> ask_datum(const std::string &label, int64_t def = 0);
     std::optional<Coordi> ask_datum_coord(const std::string &label, Coordi def = Coordi());
     std::optional<std::pair<Coordi, std::pair<bool, bool>>> ask_datum_coord2(const std::string &label,
                                                                              Coordi def = Coordi());
-    std::optional<std::string> ask_datum_string_multiline(const std::string &label, const std::string &def);
     std::optional<std::string> ask_datum_string(const std::string &label, const std::string &def);
     std::optional<int> ask_datum_angle(const std::string &label, int def = 0);
     bool edit_shapes(std::set<class Shape *> shapes);
@@ -64,6 +64,7 @@ public:
     std::optional<std::string> ask_kicad_package_filename();
     std::optional<std::string> ask_picture_filename();
     bool manage_ports(class Block &block);
+    bool update_plane(class Board &brd, class Plane *plane);
 
     class SymbolPinNamesWindow *show_symbol_pin_names_window(class SchematicSymbol &symbol);
     class RenumberPadsWindow *show_renumber_pads_window(class Package &pkg, const std::set<UUID> &pads);
@@ -75,6 +76,8 @@ public:
     class PadParameterSetWindow *show_pad_parameter_set_window(std::set<class Pad *> &pads, class IPool &pool,
                                                                class Package &pkg);
     class AlignAndDistributeWindow *show_align_and_distribute_window();
+    class EditPlaneWindow *show_edit_plane_window(class Plane &plane, class Board &brd);
+    class EditTextWindow *show_edit_text_window(class Text &text, bool use_ok);
 
     void close_nonmodal();
     class ToolWindow *get_nonmodal();
