@@ -1,7 +1,5 @@
 #pragma once
 #include <gtkmm.h>
-#include <array>
-#include <set>
 #include "util/uuid.hpp"
 #include "tool_window.hpp"
 
@@ -9,12 +7,12 @@ namespace horizon {
 
 class SymbolPinNamesWindow : public ToolWindow {
 public:
-    SymbolPinNamesWindow(Gtk::Window *parent, class ImpInterface *intf, class SchematicSymbol *s);
+    SymbolPinNamesWindow(Gtk::Window *parent, class ImpInterface *intf, class SchematicSymbol &s);
     void go_to_pin(const UUID &uu);
     UUID get_selected_pin();
 
 private:
-    class SchematicSymbol *sym = nullptr;
+    class SchematicSymbol &sym;
     class GatePinEditor *editor = nullptr;
     void handle_import();
 };
