@@ -3,6 +3,6 @@
 #include "util/placement.hpp"
 
 namespace horizon {
-void render_picture(PoDoFo::PdfDocument &doc, PoDoFo::PdfPainterMM &painter, const class Picture &pic,
+void render_picture(PoDoFo::PdfDocument &doc, PoDoFo::PdfPainter &painter, const class Picture &pic,
                     const Placement &tr = Placement());
 }
