@@ -340,6 +340,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Highlight tag", ActionGroup::GROUP_TAG, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_SPECIFIC}},
 
+        {{ActionID::TOOL, ToolID::SET_TAGS_FROM_REFDES},
+         {"Set tags from ref. desig.", ActionGroup::GROUP_TAG, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::SEARCH, ToolID::NONE},
          {"Search", ActionGroup::SEARCH,
           ActionCatalogItem::AVAILABLE_IN_PACKAGE | ActionCatalogItem::AVAILABLE_IN_SYMBOL
@@ -358,13 +362,6 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
                   | ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
-        {{ActionID::TOOL, ToolID::COPY_PLACEMENT},
-         {"Copy placement", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
-          ActionCatalogItem::FLAGS_DEFAULT}},
-
-        {{ActionID::TOOL, ToolID::COPY_TRACKS},
-         {"Copy tracks", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
-
         {{ActionID::TOOL, ToolID::SMASH},
          {"Smash", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -444,6 +441,11 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
 
         {{ActionID::TOOL, ToolID::ASSIGN_PART},
          {"Assign part", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_NO_MENU
+                  | ActionCatalogItem::FLAGS_NO_PREFERENCES}},
+
+        {{ActionID::TOOL, ToolID::CLEAR_PART},
+         {"Clear part", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::EDIT_SCHEMATIC_PROPERTIES},
@@ -502,7 +504,7 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::ADD_PLANE},
-         {"Add plane", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+         {"Assign plane", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::EDIT_PLANE},
          {"Edit plane", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
@@ -675,7 +677,7 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::ADD_KEEPOUT},
-         {"Add keepout", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_IN_PACKAGE_AND_BOARD,
+         {"Assign keepout", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_IN_PACKAGE_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::EDIT_KEEPOUT},
@@ -1063,11 +1065,51 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
         {{ActionID::TOGGLE_PICTURES, ToolID::NONE},
          {"Toggle pictures", ActionGroup::VIEW,
           ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD | ActionCatalogItem::AVAILABLE_IN_PACKAGE,
-          ActionCatalogItem::FLAGS_DEFAULT}},
+          ActionCatalogItem::FLAGS_IN_TOOL}},
 
         {{ActionID::VIEW_ACTUAL_SIZE, ToolID::NONE},
          {"View at actual size", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::OPEN_DATASHEET, ToolID::NONE},
+         {"Open datasheet", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::TOOL, ToolID::MOVE_TRACK_CENTER},
+         {"Move track arc center", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::SYMBOL_TEXT_PLACEMENT, ToolID::NONE},
+         {"Edit text placement", ActionGroup::SYMBOL, ActionCatalogItem::AVAILABLE_IN_SYMBOL,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::CYCLE_LAYER_DISPLAY_MODE, ToolID::NONE},
+         {"Cycle layer display mode", ActionGroup::LAYER, ActionCatalogItem::AVAILABLE_LAYERED,
+          ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::OPEN_PROJECT, ToolID::NONE},
+         {"Open project", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::CONVERT_TO_PAD, ToolID::NONE},
+         {"Convert to pad", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::ASSIGN_PART, ToolID::NONE},
+         {"Assign part", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::TOGGLE_SNAP_TO_PAD_BBOX, ToolID::NONE},
+         {"Toggle snap to pad corners", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PASTE_PLACEMENT},
+         {"Paste placement", ActionGroup::CLIPBOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PASTE_RELATIVE},
+         {"Paste relative", ActionGroup::CLIPBOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
@@ -1218,6 +1260,13 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(MSD_TUNING_WINDOW),
         ACTION_LUT_ITEM(TOGGLE_PICTURES),
         ACTION_LUT_ITEM(VIEW_ACTUAL_SIZE),
+        ACTION_LUT_ITEM(OPEN_DATASHEET),
+        ACTION_LUT_ITEM(SYMBOL_TEXT_PLACEMENT),
+        ACTION_LUT_ITEM(CYCLE_LAYER_DISPLAY_MODE),
+        ACTION_LUT_ITEM(OPEN_PROJECT),
+        ACTION_LUT_ITEM(CONVERT_TO_PAD),
+        ACTION_LUT_ITEM(ASSIGN_PART),
+        ACTION_LUT_ITEM(TOGGLE_SNAP_TO_PAD_BBOX),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -1262,6 +1311,7 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(PLACE_PAD),
         TOOL_LUT_ITEM(PASTE),
         TOOL_LUT_ITEM(ASSIGN_PART),
+        TOOL_LUT_ITEM(CLEAR_PART),
         TOOL_LUT_ITEM(MAP_PACKAGE),
         TOOL_LUT_ITEM(DRAW_TRACK),
         TOOL_LUT_ITEM(PLACE_VIA),
@@ -1315,8 +1365,7 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(CLEAR_TAG),
         TOOL_LUT_ITEM(RENAME_TAG),
         TOOL_LUT_ITEM(TOGGLE_GROUP_TAG_VISIBLE),
-        TOOL_LUT_ITEM(COPY_PLACEMENT),
-        TOOL_LUT_ITEM(COPY_TRACKS),
+        TOOL_LUT_ITEM(SET_TAGS_FROM_REFDES),
         TOOL_LUT_ITEM(TUNE_TRACK),
         TOOL_LUT_ITEM(TUNE_DIFFPAIR),
         TOOL_LUT_ITEM(TUNE_DIFFPAIR_SKEW),
@@ -1387,5 +1436,9 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(DRAW_NET_TIE),
         TOOL_LUT_ITEM(FLIP_NET_TIE),
         TOOL_LUT_ITEM(MOVE_TRACK_CONNECTION),
+        TOOL_LUT_ITEM(MOVE_TRACK_CENTER),
+        TOOL_LUT_ITEM(SELECT_CONNECTED_LINES),
+        TOOL_LUT_ITEM(PASTE_PLACEMENT),
+        TOOL_LUT_ITEM(PASTE_RELATIVE),
 };
 } // namespace horizon
