@@ -121,5 +121,12 @@ enum class ActionID {
     MSD_TUNING_WINDOW,
     TOGGLE_PICTURES,
     VIEW_ACTUAL_SIZE,
+    OPEN_DATASHEET,
+    SYMBOL_TEXT_PLACEMENT,
+    CYCLE_LAYER_DISPLAY_MODE,
+    OPEN_PROJECT,
+    CONVERT_TO_PAD,
+    ASSIGN_PART,
+    TOGGLE_SNAP_TO_PAD_BBOX,
 };
 }
