@@ -11,8 +11,6 @@ ClipboardHandler::ClipboardHandler(ClipboardBase &cl) : clip(cl)
 
 void ClipboardHandler::copy(std::set<SelectableRef> selection, const Coordi &cp)
 {
-    std::cout << "copy" << std::endl;
-
     auto j = clip.process(selection);
     j["cursor_pos"] = cp.as_array();
     clipboard_serialized = j.dump();
@@ -30,14 +28,11 @@ void ClipboardHandler::copy(std::set<SelectableRef> selection, const Coordi &cp)
 void ClipboardHandler::on_clipboard_get(Gtk::SelectionData &selection_data, guint /* info */)
 {
     const std::string target = selection_data.get_target();
-    std::cout << "get target " << target << std::endl;
     if (target == "imp-buffer") {
         selection_data.set("imp-buffer", clipboard_serialized);
     }
 }
 void ClipboardHandler::on_clipboard_clear()
 {
-    // buffer.clear();
-    std::cout << "clipboard clear" << std::endl;
 }
 } // namespace horizon
