@@ -6,13 +6,13 @@
 #include "logger/logger.hpp"
 #include "util/str_util.hpp"
 #include "document/idocument_package.hpp"
-#include "pool/pool.hpp"
+#include "pool/ipool.hpp"
 #include "canvas/canvas_gl.hpp"
 
 namespace horizon {
 static const auto PLUSMINUS = std::string("\u00B1");
 
-FootagDisplay *FootagDisplay::create(IDocumentPackage *c, enum footag_type type)
+FootagDisplay *FootagDisplay::create(IDocumentPackage &c, enum footag_type type)
 {
     FootagDisplay *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
@@ -22,7 +22,7 @@ FootagDisplay *FootagDisplay::create(IDocumentPackage *c, enum footag_type type)
     return w;
 }
 
-FootagDisplay::FootagDisplay(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, IDocumentPackage *c,
+FootagDisplay::FootagDisplay(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, IDocumentPackage &c,
                              enum footag_type type)
     : Gtk::Box(cobject), core(c), ppkg(UUID::random())
 {
@@ -33,7 +33,7 @@ FootagDisplay::FootagDisplay(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
 
     params = footag_get_param(ctx);
 
-    canvas_package = Gtk::manage(new PreviewCanvas(*core->get_pool(), true));
+    canvas_package = Gtk::manage(new PreviewCanvas(core.get_pool(), true));
     canvas_package->set_size_request(400, 500);
     canvas_package->signal_size_allocate().connect([this](auto &alloc) {
         if (autofit->get_active() && !(alloc == old_alloc)) {
@@ -340,7 +340,7 @@ static const char *padstack_names[] = {
 
 static_assert(sizeof(padstack_names) / sizeof(padstack_names[0]) == FOOTAG_PADSTACK_NUM, "sizes dont match");
 
-static const Padstack *getpadstack(Pool &pool, enum footag_padstack stack)
+static const Padstack *getpadstack(IPool &pool, enum footag_padstack stack)
 {
     const std::string ps_name = padstack_names[stack];
     auto ps = pool.get_well_known_padstack(ps_name);
@@ -360,7 +360,7 @@ static void make_rlimit_rect(Polygon &poly, const struct footag_rlimit *r, enum
     poly.layer = layer;
 }
 
-void FootagDisplay::calc(Package *pkg, const struct footag_spec *s)
+void FootagDisplay::calc(Package &pkg, const struct footag_spec *s)
 {
     for (int i = 0; i < s->npads; i++) {
         auto uu = UUID::random();
@@ -368,11 +368,11 @@ void FootagDisplay::calc(Package *pkg, const struct footag_spec *s)
         if (p->stack == FOOTAG_PADSTACK_NONE) {
             continue;
         }
-        auto ps = getpadstack(*core->get_pool(), p->stack);
+        auto ps = getpadstack(core.get_pool(), p->stack);
         if (!ps) {
             continue;
         }
-        auto &pad = pkg->pads.emplace(uu, Pad(uu, ps)).first->second;
+        auto &pad = pkg.pads.emplace(uu, Pad(uu, ps)).first->second;
         pad.placement.shift.x = p->x * 1_mm;
         pad.placement.shift.y = p->y * 1_mm;
         pad.name = p->name;
@@ -394,13 +394,13 @@ void FootagDisplay::calc(Package *pkg, const struct footag_spec *s)
 
     {
         auto uu = UUID::random();
-        auto &poly = pkg->polygons.emplace(uu, uu).first->second;
+        auto &poly = pkg.polygons.emplace(uu, uu).first->second;
         make_rlimit_rect(poly, &s->body, BoardLayers::TOP_PACKAGE);
     }
 
     {
         auto uu = UUID::random();
-        auto &poly = pkg->polygons.emplace(uu, uu).first->second;
+        auto &poly = pkg.polygons.emplace(uu, uu).first->second;
         make_rlimit_rect(poly, &s->courtyard, BoardLayers::TOP_COURTYARD);
         poly.parameter_class = "courtyard";
     }
@@ -415,7 +415,7 @@ void FootagDisplay::calc_and_display(void)
         return;
     }
 
-    calc(&ppkg, s);
+    calc(ppkg, s);
 
     display();
     if (s->ref.doc) {
@@ -446,7 +446,7 @@ void FootagDisplay::display(void)
 
 bool FootagDisplay::generate(void)
 {
-    auto pkg = core->get_package();
+    auto &pkg = core.get_package();
     auto s = footag_get_spec(ctx);
     if (!s) {
         return false;
