@@ -36,6 +36,8 @@
 #include "action_icon.hpp"
 #include "grids_window.hpp"
 #include "widgets/msd_tuning_window.hpp"
+#include "pool/pool_manager.hpp"
+#include "core/tools/tool_data_pool_updated.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -226,6 +228,8 @@ bool ImpBase::handle_close(const GdkEventAny *ev)
         case 2:
             if (force_end_tool()) {
                 trigger_action(ActionID::SAVE);
+                if (core->get_needs_save())
+                    return true; // saving didn't work, keep open
                 core->delete_autosave();
                 return false; // close
             }
@@ -527,7 +531,12 @@ void ImpBase::run(int argc, char *argv[])
     connect_action(ActionID::SELECTION_FILTER, [this](const auto &a) { selection_filter_dialog->present(); });
     connect_action(ActionID::SAVE, [this](const auto &a) {
         if (!read_only) {
+            if (core->get_filename().empty()) {
+                if (!set_filename())
+                    return;
+            }
             core->save();
+            save_button->set_label("Save");
             json j;
             this->get_save_meta(j);
             if (!j.is_null()) {
@@ -677,7 +686,9 @@ void ImpBase::run(int argc, char *argv[])
         set_action_sensitive(ActionID::SELECT_GRID, grids_window->has_grids());
     }
 
-    auto save_button = create_action_button(ActionID::SAVE);
+    save_button = create_action_button(ActionID::SAVE);
+    if (temp_mode)
+        save_button->set_label("Save as…");
     save_button->show();
     main_window->header->pack_start(*save_button);
     core->signal_needs_save().connect([this](bool v) {
@@ -1033,7 +1044,7 @@ void ImpBase::handle_drag()
         return;
     auto pos = canvas->get_cursor_pos_win();
     auto delta = pos - cursor_pos_drag_begin;
-    if (delta.mag_sq() > (50 * 50)) {
+    if (delta.mag() > 10) {
         clear_highlights();
         update_highlights();
         ToolArgs args;
@@ -1105,6 +1116,7 @@ void ImpBase::apply_preferences()
         it->set_keep_primary_action(!preferences.action_bar.remember);
     }
     main_window->set_use_action_bar(preferences.action_bar.enable);
+    main_window->tool_bar_set_vertical(preferences.tool_bar.vertical_layout);
     core->set_history_max(preferences.undo_redo.max_depth);
     preferences_apply_appearance(preferences);
 }
@@ -1350,18 +1362,27 @@ bool ImpBase::handle_click(const GdkEventButton *button_event)
 
     bool need_menu = false;
     if (core->tool_is_active() && button_event->button != 2 && !(button_event->state & Gdk::SHIFT_MASK)
-        && button_event->type != GDK_2BUTTON_PRESS && button_event->type != GDK_3BUTTON_PRESS) {
-        ToolArgs args;
-        args.type = ToolEventType::ACTION;
-        if (button_event->button == 1)
-            args.action = InToolActionID::LMB;
-        else
-            args.action = InToolActionID::RMB;
-        args.coords = canvas->get_cursor_pos();
-        args.target = canvas->get_current_target();
-        args.work_layer = canvas->property_work_layer();
-        ToolResponse r = core->tool_update(args);
-        tool_process(r);
+        && button_event->type != GDK_3BUTTON_PRESS) {
+        const bool is_doubleclick = button_event->type == GDK_2BUTTON_PRESS;
+        const bool tool_supports_doubleclick = core->get_tool_actions().count(InToolActionID::LMB_DOUBLE);
+        if (tool_supports_doubleclick || !is_doubleclick) {
+            ToolArgs args;
+            args.type = ToolEventType::ACTION;
+            if (button_event->button == 1) {
+                if (is_doubleclick)
+                    args.action = InToolActionID::LMB_DOUBLE;
+                else
+                    args.action = InToolActionID::LMB;
+            }
+            else {
+                args.action = InToolActionID::RMB;
+            }
+            args.coords = canvas->get_cursor_pos();
+            args.target = canvas->get_current_target();
+            args.work_layer = canvas->property_work_layer();
+            ToolResponse r = core->tool_update(args);
+            tool_process(r);
+        }
     }
     else if (!core->tool_is_active() && button_event->type == GDK_2BUTTON_PRESS) {
         auto sel = canvas->get_selection();
@@ -1473,7 +1494,7 @@ void ImpBase::handle_maybe_drag(bool ctrl)
     }
 }
 
-void ImpBase::tool_process(ToolResponse &resp)
+void ImpBase::tool_process_one()
 {
     if (!core->tool_is_active()) {
         imp_interface->dialogs.close_nonmodal();
@@ -1494,6 +1515,19 @@ void ImpBase::tool_process(ToolResponse &resp)
                                   canvas->get_selection_mode() == CanvasGL::SelectionMode::NORMAL);
         }
     }
+}
+
+void ImpBase::tool_process(ToolResponse &resp)
+{
+    tool_process_one();
+    while (auto args = core->get_pending_tool_args()) {
+        auto r = core->tool_update(*args);
+        if (r.next_tool != ToolID::NONE) {
+            Logger::log_critical("shouldn't have received next tool in deferred tool_update", Logger::Domain::CORE);
+            return;
+        }
+        tool_process_one();
+    }
     if (resp.next_tool != ToolID::NONE) {
         clear_highlights();
         update_highlights();
@@ -1611,6 +1645,17 @@ bool ImpBase::handle_broadcast(const json &j)
         preferences.signal_changed().emit();
         return true;
     }
+    else if (op == "reload-pools") {
+        PoolManager::get().reload();
+        return true;
+    }
+    else if (op == "pool-updated") {
+        const auto path = j.at("path").get<std::string>();
+        if (core->tool_is_active() && path == pool->get_base_path()) {
+            tool_update_data(std::make_unique<ToolDataPoolUpdated>());
+        }
+        return false; // so that derived imps can handle this as well
+    }
     return false;
 }
 
@@ -1630,7 +1675,7 @@ void ImpBase::set_monitor_items(const ItemSet &items)
 {
     std::set<std::string> filenames;
     std::transform(items.begin(), items.end(), std::inserter(filenames, filenames.begin()),
-                   [this](const auto &it) { return pool_caching->get_filename(it.first, it.second); });
+                   [this](const auto &it) { return pool->get_filename(it.first, it.second); });
     file_monitor_delay_connection.disconnect();
     file_monitor_delay_connection = Glib::signal_timeout().connect_seconds(
             [this, filenames] {
@@ -1653,7 +1698,7 @@ void ImpBase::set_read_only(bool v)
     read_only = v;
 }
 
-void ImpBase::tool_update_data(std::unique_ptr<ToolData> &data)
+void ImpBase::tool_update_data(std::unique_ptr<ToolData> data)
 {
 
     if (core->tool_is_active()) {
@@ -1899,4 +1944,16 @@ void ImpBase::update_cursor(ToolID tool_id)
     cr->paint();
     win->set_cursor(Gdk::Cursor::create(win->get_display(), surf, 0, 0));
 }
+
+bool ImpBase::set_filename()
+{
+    throw std::runtime_error("set_filename not implemented");
+    return false;
+}
+
+void ImpBase::set_suggested_filename(const std::string &s)
+{
+    suggested_filename = s;
+}
+
 } // namespace horizon
