@@ -46,6 +46,8 @@ public:
     {
     }
     void set_read_only(bool v);
+    void set_suggested_filename(const std::string &s);
+    enum class TempMode { YES, NO };
 
     class SelectionFilterInfo {
     public:
@@ -178,6 +180,7 @@ protected:
     std::string get_hud_text_for_component(const Component *comp);
     virtual const Block &get_block_for_group_tag_names();
     std::string get_hud_text_for_net(const Net *net);
+    std::string get_hud_text_for_layers(const std::set<int> &layers);
 
     void set_monitor_files(const std::set<std::string> &files);
     void set_monitor_items(const ItemSet &items);
@@ -190,7 +193,7 @@ protected:
 
     bool read_only = false;
 
-    void tool_update_data(std::unique_ptr<ToolData> &data);
+    void tool_update_data(std::unique_ptr<ToolData> data);
 
     virtual void search_center(const Searcher::SearchResult &res);
     virtual ActionToolID get_doubleclick_action(ObjectType type, const UUID &uu);
@@ -261,6 +264,9 @@ protected:
         throw std::runtime_error("not implemented");
     }
 
+    bool temp_mode = false;
+    std::string suggested_filename;
+
 private:
     void fix_cursor_pos();
     Glib::RefPtr<Gio::FileMonitor> preferences_monitor;
@@ -329,6 +335,8 @@ private:
     InToolKeySequencesPreferences in_tool_key_sequeces_preferences;
     std::vector<ActionLabelInfo> in_tool_action_label_infos;
 
+    void tool_process_one();
+
     void show_preferences(std::optional<std::string> page = std::nullopt);
 
     void init_search();
@@ -354,5 +362,8 @@ private:
     unsigned int saved_version = 0;
 
     class MSDTuningWindow *msd_tuning_window = nullptr;
+
+    Gtk::Button *save_button = nullptr;
+    virtual bool set_filename();
 };
 } // namespace horizon
