@@ -29,6 +29,11 @@ void ImpInterface::tool_bar_flash(const std::string &s)
     imp->main_window->tool_bar_flash(s);
 }
 
+void ImpInterface::tool_bar_flash_replace(const std::string &s)
+{
+    imp->main_window->tool_bar_flash_replace(s);
+}
+
 void ImpInterface::part_placed(const UUID &uu)
 {
     imp->send_json({{"op", "part-placed"}, {"part", (std::string)uu}});
@@ -76,7 +81,7 @@ std::set<ObjectRef> &ImpInterface::get_highlights()
 
 void ImpInterface::tool_update_data(std::unique_ptr<ToolData> data)
 {
-    imp->tool_update_data(data);
+    imp->tool_update_data(std::move(data));
 }
 
 void ImpInterface::set_layer_display(int layer, const class LayerDisplay &ld)
@@ -134,5 +139,12 @@ Coordi ImpInterface::transform_arrow_keys(const Coordi &c) const
     }
 }
 
+void ImpInterface::pool_update(const std::vector<std::string> &filenames)
+{
+    json j;
+    j["op"] = "update-pool";
+    j["filenames"] = filenames;
+    imp->send_json(j);
+}
 
 } // namespace horizon
