@@ -21,10 +21,9 @@
 #include <iostream>
 
 namespace horizon {
-ImpSchematic::ImpSchematic(const std::string &blocks_filename, const std::string &pictures_dir,
-                           const PoolParams &pool_params)
-    : ImpBase(pool_params), core_schematic(blocks_filename, pictures_dir, *pool, *pool_caching),
-      project_dir(Glib::path_get_dirname(blocks_filename)), searcher(core_schematic)
+ImpSchematic::ImpSchematic(const CoreSchematic::Filenames &filenames, const PoolParams &pool_params)
+    : ImpBase(pool_params), core_schematic(filenames, *pool, *pool_caching),
+      project_dir(Glib::path_get_dirname(filenames.blocks)), searcher(core_schematic)
 {
     core = &core_schematic;
     core_schematic.signal_tool_changed().connect(sigc::mem_fun(*this, &ImpSchematic::handle_tool_change));
@@ -116,7 +115,7 @@ public:
         set_focus_on_click(false);
         signal_clicked().connect([this] { s_signal_selected.emit(path, sheet); });
         signal_enter_notify_event().connect([this](GdkEventCrossing *ev) -> bool {
-            get_window()->set_cursor(Gdk::Cursor::create(Gdk::Display::get_default(), "pointer"));
+            get_window()->set_cursor(Gdk::Cursor::create(get_window()->get_display(), "pointer"));
             return false;
         });
         signal_leave_notify_event().connect([this](GdkEventCrossing *ev) -> bool {
@@ -611,6 +610,41 @@ void ImpSchematic::construct()
             this->send_json(j);
         }
     });
+    connect_action(ActionID::ASSIGN_PART, [this](const auto &a) {
+        const auto sel = canvas->get_selection();
+
+        if (!sockets_connected) {
+            this->tool_begin(ToolID::ASSIGN_PART);
+        }
+        else if (auto entity = entity_from_selection(sel)) {
+            canvas->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
+            handle_selection_cross_probe();
+            std::set<const Component *> components;
+            const auto &sheet = *core_schematic.get_sheet();
+            for (const auto &it : sel) {
+                if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
+                    if (sheet.symbols.count(it.uuid))
+                        components.insert(sheet.symbols.at(it.uuid).component);
+                }
+            }
+            std::string hint;
+            for (const auto comp : components) {
+                if (hint.size())
+                    hint += ", ";
+                hint += core_schematic.get_top_block()
+                                ->get_component_info(*comp, core_schematic.get_instance_path())
+                                .refdes;
+            }
+            hint = "Assigning to " + hint;
+            json j;
+            j["op"] = "show-browser-assign";
+            j["time"] = gtk_get_current_event_time();
+            j["entity"] = (std::string)entity->uuid;
+            j["hint"] = hint;
+            allow_set_foreground_window(mgr_pid);
+            this->send_json(j);
+        }
+    });
 
     connect_action(ActionID::PREV_SHEET, sigc::mem_fun(*this, &ImpSchematic::handle_next_prev_sheet));
     connect_action(ActionID::NEXT_SHEET, sigc::mem_fun(*this, &ImpSchematic::handle_next_prev_sheet));
@@ -869,6 +903,16 @@ void ImpSchematic::construct()
         }
     });
 
+    connect_action(ActionID::OPEN_DATASHEET, [this](const auto &a) {
+        const auto x = sel_find_exactly_one(canvas->get_selection(), ObjectType::SCHEMATIC_SYMBOL);
+        if (x && core_schematic.get_sheet()->symbols.count(x->uuid)) {
+            auto part = core_schematic.get_sheet()->symbols.at(x->uuid).component->part;
+            if (part && part->get_datasheet().size())
+                gtk_show_uri_on_window(GTK_WINDOW(main_window->gobj()), part->get_datasheet().c_str(), GDK_CURRENT_TIME,
+                                       NULL);
+        }
+    });
+
     add_action_button_schematic(ActionID::PLACE_PART);
     add_action_button_schematic(ToolID::DRAW_NET);
     {
@@ -890,9 +934,33 @@ ActionButton &ImpSchematic::add_action_button_schematic(ActionToolID id)
     return b;
 }
 
+const Entity *ImpSchematic::entity_from_selection(const std::set<SelectableRef> &sel)
+{
+    if (core_schematic.get_block_symbol_mode())
+        return nullptr;
+    const Entity *entity = nullptr;
+    const auto &sheet = *core_schematic.get_sheet();
+    for (const auto &it : sel) {
+        if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
+            if (!sheet.symbols.count(it.uuid))
+                continue;
+            const auto &sym = sheet.symbols.at(it.uuid);
+            if (entity) {
+                if (entity != sym.component->entity) {
+                    return nullptr;
+                }
+            }
+            else {
+                entity = sym.component->entity;
+            }
+        }
+    }
+    return entity;
+}
+
 void ImpSchematic::update_action_sensitivity()
 {
-    auto sel = canvas->get_selection();
+    const auto sel = canvas->get_selection();
     const auto n_block_sym = sel_count_type(sel, ObjectType::SCHEMATIC_BLOCK_SYMBOL);
     if (sockets_connected) {
         json req;
@@ -946,6 +1014,16 @@ void ImpSchematic::update_action_sensitivity()
         }
     }
 
+    {
+        const auto x = sel_find_exactly_one(sel, ObjectType::SCHEMATIC_SYMBOL);
+        bool sensitive = false;
+        if (x && core_schematic.get_sheet()->symbols.count(x->uuid)) {
+            auto part = core_schematic.get_sheet()->symbols.at(x->uuid).component->part;
+            sensitive = x && part && part->get_datasheet().size();
+        }
+        set_action_sensitive(ActionID::OPEN_DATASHEET, sensitive);
+    }
+
     set_action_sensitive(ActionID::NEXT_SHEET, !core_schematic.get_block_symbol_mode());
     set_action_sensitive(ActionID::PREV_SHEET, !core_schematic.get_block_symbol_mode());
 
@@ -957,6 +1035,9 @@ void ImpSchematic::update_action_sensitivity()
     set_action_sensitive(ActionID::EDIT_BLOCK_SYMBOL, have_block_sym);
     set_action_sensitive(ActionID::POP_OUT_OF_BLOCK, core_schematic.get_instance_path().size());
     set_action_sensitive(ActionID::GO_TO_BLOCK_SYMBOL, !core_schematic.current_block_is_top());
+
+    set_action_sensitive(ActionID::ASSIGN_PART, entity_from_selection(sel));
+
     ImpBase::update_action_sensitivity();
 }
 
@@ -1458,6 +1539,10 @@ void ImpSchematic::handle_move_to_other_sheet(const ActionConnection &conn)
             new_sheet->arcs.insert(std::make_pair(it.uuid, std::move(old_sheet->arcs.at(it.uuid))));
             old_sheet->arcs.erase(it.uuid);
         } break;
+        case ObjectType::PICTURE: {
+            new_sheet->pictures.insert(std::make_pair(it.uuid, std::move(old_sheet->pictures.at(it.uuid))));
+            old_sheet->pictures.erase(it.uuid);
+        } break;
 
         default:;
         }
