@@ -7,7 +7,7 @@ namespace horizon {
 class ImpSchematic : public ImpBase {
 
 public:
-    ImpSchematic(const std::string &blocks_filename, const std::string &pictures_dir, const PoolParams &params);
+    ImpSchematic(const CoreSchematic::Filenames &filenames, const PoolParams &params);
 
 protected:
     void construct() override;
@@ -53,6 +53,7 @@ private:
     void handle_move_to_other_sheet(const ActionConnection &conn);
     void handle_highlight_group_tag(const ActionConnection &conn);
     void handle_next_prev_sheet(const ActionConnection &conn);
+    const Entity *entity_from_selection(const std::set<SelectableRef> &sel);
 
     struct ViewInfo {
         float scale;
