@@ -7,6 +7,7 @@ enum class InToolActionID {
     // common
     LMB,
     LMB_RELEASE,
+    LMB_DOUBLE,
     RMB,
     ROTATE,
     ROTATE_CURSOR,
@@ -69,6 +70,8 @@ enum class InToolActionID {
     CLEARANCE_OFFSET,
     CLEARANCE_OFFSET_DEFAULT,
     ROUTER_MODE,
+    DELETE_LAST_SEGMENT,
+    TOGGLE_CORNER_STYLE,
 
     // set NC
     NC_MODE,
