@@ -23,6 +23,7 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     GET_WIDGET(tool_bar_stack);
     GET_WIDGET(tool_bar_tip_label);
     GET_WIDGET(tool_bar_flash_label);
+    GET_WIDGET(tool_bar_box);
     GET_WIDGET(selection_label);
     GET_WIDGET(header);
     GET_WIDGET(property_scrolled_window);
@@ -58,7 +59,6 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     GET_WIDGET(grid_reset_origin_button);
     GET_WIDGET(tool_bar_action_tip_label);
     {
-        Gtk::Box *tool_bar_actions_box = nullptr;
         GET_WIDGET(tool_bar_actions_box);
         tool_bar_actions_reflow_box = Gtk::manage(new ReflowBox());
         tool_bar_actions_reflow_box->property_spacing() = 10;
@@ -118,6 +118,7 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
 
     label_set_tnum(cursor_label);
     label_set_tnum(tool_bar_tip_label);
+    label_set_tnum(tool_bar_flash_label);
     label_set_tnum(grid_mul_label);
     label_set_tnum(search_status_label);
     {
@@ -161,7 +162,13 @@ void MainWindow::tool_bar_set_tool_name(const std::string &s)
 void MainWindow::tool_bar_set_tool_tip(const std::string &s)
 {
     if (tool_bar_use_actions) {
-        tool_bar_action_tip_label->set_markup(s);
+        if (s.size()) {
+            tool_bar_action_tip_label->set_markup(s);
+            tool_bar_action_tip_label->show();
+        }
+        else {
+            tool_bar_action_tip_label->hide();
+        }
     }
     else {
         tool_bar_tip_label->set_markup(s);
@@ -170,7 +177,17 @@ void MainWindow::tool_bar_set_tool_tip(const std::string &s)
 
 void MainWindow::tool_bar_flash(const std::string &s)
 {
-    if (flash_text.size())
+    tool_bar_flash(s, false);
+}
+
+void MainWindow::tool_bar_flash_replace(const std::string &s)
+{
+    tool_bar_flash(s, true);
+}
+
+void MainWindow::tool_bar_flash(const std::string &s, bool replace)
+{
+    if (flash_text.size() && !replace)
         flash_text += "; " + s;
     else
         flash_text = s;
@@ -195,6 +212,11 @@ void MainWindow::tool_bar_flash(const std::string &s)
             2000);
 }
 
+void MainWindow::tool_bar_set_vertical(bool v)
+{
+    tool_bar_box->set_orientation(v ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL);
+}
+
 void MainWindow::hud_update(const std::string &s)
 {
     hud_timeout_connection.disconnect();
@@ -279,12 +301,14 @@ void MainWindow::tool_bar_clear_actions()
 {
     for (auto ch : tool_bar_actions_reflow_box->get_children())
         delete ch;
+    tool_bar_actions_box->hide();
 }
 
 void MainWindow::tool_bar_append_action(Gtk::Widget &w)
 {
     w.show();
     tool_bar_actions_reflow_box->append(w);
+    tool_bar_actions_box->show();
 }
 
 void MainWindow::set_version_info(const std::string &s)
