@@ -68,9 +68,11 @@ public:
     void tool_bar_set_tool_name(const std::string &s);
     void tool_bar_set_tool_tip(const std::string &s);
     void tool_bar_flash(const std::string &s);
+    void tool_bar_flash_replace(const std::string &s);
     void tool_bar_set_use_actions(bool use_actions);
     void tool_bar_clear_actions();
     void tool_bar_append_action(Gtk::Widget &w);
+    void tool_bar_set_vertical(bool vert);
 
     void hud_update(const std::string &s);
     void hud_hide();
@@ -96,10 +98,13 @@ private:
     Gtk::Label *tool_bar_flash_label = nullptr;
     Gtk::Stack *tool_bar_stack = nullptr;
     Gtk::Label *tool_bar_action_tip_label = nullptr;
+    Gtk::Box *tool_bar_box = nullptr;
+    Gtk::Box *tool_bar_actions_box = nullptr;
     class ReflowBox *tool_bar_actions_reflow_box = nullptr;
     sigc::connection tip_timeout_connection;
     std::string flash_text;
     bool tool_bar_queue_close = false;
+    void tool_bar_flash(const std::string &s, bool replace);
 
     Gtk::Revealer *hud = nullptr;
     Gtk::Label *hud_label = nullptr;
