@@ -6,12 +6,10 @@
 namespace horizon {
 RulePackageChecks::RulePackageChecks() : Rule()
 {
-    id = RuleID::PACKAGE_CHECKS;
 }
 
 RulePackageChecks::RulePackageChecks(const json &j) : Rule(j)
 {
-    id = RuleID::PACKAGE_CHECKS;
 }
 
 json RulePackageChecks::serialize() const
@@ -20,7 +18,7 @@ json RulePackageChecks::serialize() const
     return j;
 }
 
-std::string RulePackageChecks::get_brief(const class Block *block) const
+std::string RulePackageChecks::get_brief(const class Block *block, class IPool *pool) const
 {
     return "";
 }
