@@ -5,6 +5,7 @@
 #include "widgets/pool_browser_frame.hpp"
 #include "util/win32_undef.hpp"
 #include "widgets/preview_canvas.hpp"
+#include "util/gtk_util.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_edit_frame(const UUID &uu)
@@ -14,23 +15,11 @@ void PoolNotebook::handle_edit_frame(const UUID &uu)
 
 void PoolNotebook::handle_create_frame()
 {
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
-    UUID unit_uuid;
+    Frame fr(horizon::UUID::random());
+    std::string fn = pool.get_tmp_filename(ObjectType::SYMBOL, fr.uuid);
+    save_json_to_file(fn, fr.serialize());
 
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Frame", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    chooser->set_current_folder(Glib::build_filename(base_path, "frames"));
-
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = append_dot_json(chooser->get_filename());
-        Frame fr(horizon::UUID::random());
-        fr.name = "fixme";
-        save_json_to_file(fn, fr.serialize());
-        pool_update({fn});
-        appwin.spawn(PoolProjectManagerProcess::Type::IMP_FRAME, {fn});
-    }
+    appwin.spawn(PoolProjectManagerProcess::Type::IMP_FRAME, {fn}, PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_duplicate_frame(const UUID &uu)
