@@ -9,6 +9,7 @@
 #include "util/win32_undef.hpp"
 #include "widgets/preview_canvas.hpp"
 #include "import_kicad_package_window.hpp"
+#include "util/gtk_util.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_edit_package(const UUID &uu)
@@ -18,66 +19,28 @@ void PoolNotebook::handle_edit_package(const UUID &uu)
 
 void PoolNotebook::handle_create_package()
 {
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
+    Package pkg(horizon::UUID::random());
+    std::string fn = pool.get_tmp_filename(ObjectType::PACKAGE, pkg.uuid);
+    save_json_to_file(fn, pkg.serialize());
 
-    GtkFileChooserNative *native = gtk_file_chooser_native_new(
-            "Save Package", top->gobj(), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    chooser->set_current_name("package");
-    chooser->set_current_folder(Glib::build_filename(base_path, "packages"));
-
-    while (true) {
-        chooser->set_current_folder(Glib::build_filename(base_path, "packages"));
-        if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-            std::string fn = chooser->get_filename();
-            Glib::Dir dir(fn);
-            int n = 0;
-            for (const auto &it : dir) {
-                (void)it;
-                n++;
-            }
-            if (n > 0) {
-                Gtk::MessageDialog md(*top, "Folder must be empty", false /* use_markup */, Gtk::MESSAGE_ERROR,
-                                      Gtk::BUTTONS_OK);
-                md.run();
-                continue;
-            }
-            else {
-                auto fi = Gio::File::create_for_path(Glib::build_filename(fn, "padstacks"));
-                fi->make_directory_with_parents();
-                Package pkg(horizon::UUID::random());
-                auto pkg_filename = Glib::build_filename(fn, "package.json");
-                save_json_to_file(pkg_filename, pkg.serialize());
-                appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {pkg_filename});
-            }
-        }
-        break;
-    }
+    appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {fn}, PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_create_padstack_for_package(const UUID &uu)
 {
     if (!uu)
         return;
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
 
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Padstack", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    chooser->set_current_name("pad.json");
     auto pkg_filename = pool.get_filename(ObjectType::PACKAGE, uu);
     auto pkg_dir = Glib::path_get_dirname(pkg_filename);
-    chooser->set_current_folder(Glib::build_filename(pkg_dir, "padstacks"));
+    const auto padstack_dir = Glib::build_filename(pkg_dir, "padstacks");
 
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = append_dot_json(chooser->get_filename());
-        Padstack ps(horizon::UUID::random());
-        ps.name = "Pad";
-        save_json_to_file(fn, ps.serialize());
-        appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
-    }
+    Padstack padstack(horizon::UUID::random());
+    std::string fn = pool.get_tmp_filename(ObjectType::PADSTACK, padstack.uuid);
+    save_json_to_file(fn, padstack.serialize());
+
+    appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn, padstack_dir},
+                 PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_duplicate_package(const UUID &uu)
@@ -86,10 +49,9 @@ void PoolNotebook::handle_duplicate_package(const UUID &uu)
         return;
     auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
 
-    GtkFileChooserNative *native = gtk_file_chooser_native_new(
-            "Save Package", top->gobj(), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, "_Save", "_Cancel");
+    GtkFileChooserNative *native =
+            gtk_file_chooser_native_new("Save Package", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
     auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
     auto pkg_filename = Glib::build_filename(pool.get_base_path(),
                                              Glib::path_get_dirname(pool.get_rel_filename(ObjectType::PACKAGE, uu)));
     auto pkg_basename = Glib::path_get_basename(pkg_filename);
@@ -97,34 +59,23 @@ void PoolNotebook::handle_duplicate_package(const UUID &uu)
     chooser->set_current_folder(pkg_dirname);
     chooser->set_current_name(pkg_basename + "-copy");
 
-    while (true) {
-        chooser->set_current_folder(pkg_dirname);
-        if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-            std::string fn = chooser->get_filename();
 
-            Glib::Dir dir(fn);
-            int n = 0;
-            for (const auto &it : dir) {
-                (void)it;
-                n++;
-            }
-            if (n > 0) {
-                Gtk::MessageDialog md(*top, "Folder must be empty", false /* use_markup */, Gtk::MESSAGE_ERROR,
-                                      Gtk::BUTTONS_OK);
-                md.run();
-                continue;
-            }
-            else {
-                std::vector<std::string> filenames;
-                DuplicatePartWidget::duplicate_package(pool, uu, fn, Glib::path_get_basename(fn), &filenames);
-                std::string new_pkg_filename = Glib::build_filename(fn, "package.json");
-                pool_update_done_cb = [this, new_pkg_filename] {
-                    appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {new_pkg_filename});
-                };
-                pool_update(filenames);
-            }
-        }
-        break;
+    std::vector<std::string> filenames;
+    std::string new_pkg_filename;
+    auto success = run_native_filechooser_with_retry(
+            chooser, "Error creating package", [this, chooser, &filenames, &uu, &new_pkg_filename] {
+                auto filename = chooser->get_filename();
+                pool.check_filename_throw(ObjectType::PACKAGE, filename);
+                DuplicatePartWidget::duplicate_package(pool, uu, filename, Glib::path_get_basename(filename),
+                                                       &filenames);
+                new_pkg_filename = Glib::build_filename(filename, "package.json");
+            });
+
+    if (success) {
+        pool_update_done_cb = [this, new_pkg_filename] {
+            appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {new_pkg_filename});
+        };
+        pool_update(filenames);
     }
 }
 
@@ -135,10 +86,6 @@ void PoolNotebook::handle_import_kicad_package()
         import_kicad_package_window = ImportKiCadPackageWindow::create(appwin);
         import_kicad_package_window->present();
         import_kicad_package_window->signal_hide().connect([this] {
-            const auto files_saved = import_kicad_package_window->get_files_saved();
-            if (files_saved.size()) {
-                pool_update(files_saved);
-            }
             delete import_kicad_package_window;
             import_kicad_package_window = nullptr;
         });
