@@ -5,6 +5,7 @@
 #include "widgets/pool_browser_padstack.hpp"
 #include "widgets/padstack_preview.hpp"
 #include "widgets/where_used_box.hpp"
+#include "util/gtk_util.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_edit_padstack(const UUID &uu)
@@ -14,21 +15,11 @@ void PoolNotebook::handle_edit_padstack(const UUID &uu)
 
 void PoolNotebook::handle_create_padstack()
 {
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
+    Padstack padstack(horizon::UUID::random());
+    std::string fn = pool.get_tmp_filename(ObjectType::PADSTACK, padstack.uuid);
+    save_json_to_file(fn, padstack.serialize());
 
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Padstack", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    chooser->set_current_name("padstack.json");
-    chooser->set_current_folder(Glib::build_filename(base_path, "padstacks"));
-
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = append_dot_json(chooser->get_filename());
-        Padstack ps(horizon::UUID::random());
-        save_json_to_file(fn, ps.serialize());
-        appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
-    }
+    appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn}, PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_duplicate_padstack(const UUID &uu)
