@@ -1,5 +1,6 @@
 #include "pool_notebook.hpp"
 #include "util/util.hpp"
+#include "util/gtk_util.hpp"
 #include "nlohmann/json.hpp"
 #include "pool-prj-mgr/pool-prj-mgr-app_win.hpp"
 #include "widgets/pool_browser_unit.hpp"
@@ -21,68 +22,41 @@ void PoolNotebook::handle_create_symbol_for_unit(const UUID &uu)
 {
     if (!uu)
         return;
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
 
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Symbol", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    {
-        auto unit_filename = pool.get_filename(ObjectType::UNIT, uu);
-        auto unit_dirname_rel =
-                Gio::File::create_for_path(Glib::build_filename(base_path, "units"))
-                        ->get_relative_path(Gio::File::create_for_path(Glib::path_get_dirname(unit_filename)));
+    Symbol sym(horizon::UUID::random());
+    auto unit = pool.get_unit(uu);
+    sym.name = unit->name;
+    sym.unit = unit;
+    std::string fn = pool.get_tmp_filename(ObjectType::SYMBOL, sym.uuid);
+    save_json_to_file(fn, sym.serialize());
+    auto unit_filename = pool.get_filename(ObjectType::UNIT, uu);
+    auto unit_dirname_rel =
+            Gio::File::create_for_path(Glib::build_filename(base_path, "units"))
+                    ->get_relative_path(Gio::File::create_for_path(Glib::path_get_dirname(unit_filename)));
 
-        chooser->set_current_folder(Glib::build_filename(base_path, "symbols", unit_dirname_rel));
-        chooser->set_current_name(Glib::path_get_basename(unit_filename));
-    }
+    const auto suggested_filename =
+            Glib::build_filename(base_path, "symbols", unit_dirname_rel, Glib::path_get_basename(unit_filename));
 
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = append_dot_json(chooser->get_filename());
-        Symbol sym(horizon::UUID::random());
-        auto unit = pool.get_unit(uu);
-        sym.name = unit->name;
-        sym.unit = unit;
-        save_json_to_file(fn, sym.serialize());
-        pool_update({fn});
-        appwin.spawn(PoolProjectManagerProcess::Type::IMP_SYMBOL, {fn});
-    }
+    appwin.spawn(PoolProjectManagerProcess::Type::IMP_SYMBOL, {fn, suggested_filename},
+                 PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_create_entity_for_unit(const UUID &uu)
 {
     if (!uu)
         return;
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
 
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Entity", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    {
-        auto unit_filename = pool.get_filename(ObjectType::UNIT, uu);
-        auto unit_dirname_rel =
-                Gio::File::create_for_path(Glib::build_filename(base_path, "units"))
-                        ->get_relative_path(Gio::File::create_for_path(Glib::path_get_dirname(unit_filename)));
+    Entity entity(horizon::UUID::random());
+    auto unit = pool.get_unit(uu);
+    entity.name = unit->name;
+    auto uu2 = UUID::random();
+    auto gate = &entity.gates.emplace(uu2, uu2).first->second;
+    gate->unit = unit;
+    gate->name = "Main";
+    const auto filename = pool.get_tmp_filename(ObjectType::ENTITY, entity.uuid);
+    save_json_to_file(filename, entity.serialize());
 
-        chooser->set_current_folder(Glib::build_filename(base_path, "entities", unit_dirname_rel));
-        chooser->set_current_name(Glib::path_get_basename(unit_filename));
-    }
-
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = append_dot_json(chooser->get_filename());
-        Entity entity(horizon::UUID::random());
-        auto unit = pool.get_unit(uu);
-        entity.name = unit->name;
-        auto uu2 = UUID::random();
-        auto gate = &entity.gates.emplace(uu2, uu2).first->second;
-        gate->unit = unit;
-        gate->name = "Main";
-
-        save_json_to_file(fn, entity.serialize());
-        pool_update({fn});
-        appwin.spawn(PoolProjectManagerProcess::Type::ENTITY, {fn});
-    }
+    appwin.spawn(PoolProjectManagerProcess::Type::ENTITY, {filename}, PoolProjectManagerAppWindow::SpawnFlags::TEMP);
 }
 
 void PoolNotebook::handle_duplicate_unit(const UUID &uu)
