@@ -18,6 +18,7 @@
 #include "util/zmq_helper.hpp"
 #include "pools_window/pools_window.hpp"
 #include "util/stock_info_provider.hpp"
+#include "pool/pool_manager.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -101,6 +102,8 @@ void PoolProjectManagerApplication::UserConfig::load(const std::string &filename
     pool_doc_info_bar_dismissed = j.value("pool_doc_info_bar_dismissed", false);
     project_base_path = j.value("project_base_path", "");
     project_author = j.value("project_author", "");
+    if (j.count("project_pool"))
+        project_pool = j.at("project_pool").get<std::string>();
     if (j.count("recent")) {
         const json &o = j["recent"];
         for (const auto &[fn, v] : o.items()) {
@@ -126,6 +129,7 @@ void PoolProjectManagerApplication::UserConfig::save(const std::string &filename
     j["project_author"] = project_author;
     j["project_base_path"] = project_base_path;
     j["pool_doc_info_bar_dismissed"] = pool_doc_info_bar_dismissed;
+    j["project_pool"] = (std::string)project_pool;
     save_json_to_file(filename, j);
 }
 
@@ -176,6 +180,12 @@ void PoolProjectManagerApplication::on_startup()
     });
     preferences_apply_appearance(preferences);
 
+    PoolManager::get().signal_written().connect([this] {
+        json j;
+        j["op"] = "reload-pools";
+        send_json(0, j);
+    });
+
     log_window = new LogWindow();
     log_dispatcher.set_handler([this](const auto &it) { log_window->get_view()->push_log(it); });
     Logger::get().set_log_handler([this](const Logger::Item &it) { log_dispatcher.log(it); });
@@ -260,7 +270,8 @@ bool PoolProjectManagerApplication::present_existing_window(const std::string &p
     return false;
 }
 
-PoolProjectManagerAppWindow &PoolProjectManagerApplication::open_pool(const std::string &pool_json, guint32 timestamp)
+PoolProjectManagerAppWindow &PoolProjectManagerApplication::open_pool_or_project(const std::string &pool_json,
+                                                                                 guint32 timestamp)
 {
     PoolProjectManagerAppWindow *appwindow = nullptr;
     for (auto ws : get_windows()) {
