@@ -240,6 +240,12 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
                     part_browser_window->pool_updated(path);
                 if (project && project->pool_directory == path)
                     view_project.update_pools_label();
+                {
+                    json j;
+                    j["op"] = "pool-updated";
+                    j["path"] = path;
+                    app.send_json(0, j);
+                }
             },
             *this));
 
@@ -457,6 +463,13 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         part_browser_window->placed_part(part);
     }
     else if (op == "show-browser") {
+        part_browser_window->set_entity({}, "");
+        part_browser_window->present(timestamp);
+        part_browser_window->focus_search();
+    }
+    else if (op == "show-browser-assign") {
+        const UUID entity_uuid = j.at("entity").get<std::string>();
+        part_browser_window->set_entity(entity_uuid, j.at("hint").get<std::string>());
         part_browser_window->present(timestamp);
         part_browser_window->focus_search();
     }
@@ -554,12 +567,20 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
     else if (op == "needs-save") {
         const auto pid = j.at("pid").get<int>();
         const auto needs_save = j.at("needs_save").get<bool>();
+        const auto filename = j.at("filename").get<std::string>();
         std::cout << "needs save " << pid << " " << needs_save << std::endl;
+        if (!needs_save) {
+            for (auto &it : processes) {
+                if (it.second.proc && it.second.proc->get_pid() == pid) {
+                    it.second.set_filename(filename);
+                }
+            }
+        }
         pids_need_save[pid] = needs_save;
         if (project && !needs_save)
             view_project.update_meta();
         if (!needs_save) {
-            s_signal_process_saved.emit(j.at("filename").get<std::string>());
+            s_signal_process_saved.emit(filename);
             if (auto proc = find_top_schematic_process()) {
                 if (proc->proc->get_pid() == pid) { // schematic got saved
                     if (auto proc_board = find_board_process()) {
@@ -627,7 +648,7 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             pool_path = pool2->base_path;
         }
         if (pool_path.size()) {
-            auto &win = app.open_pool(Glib::build_filename(pool_path, "pool.json"), timestamp);
+            auto &win = app.open_pool_or_project(Glib::build_filename(pool_path, "pool.json"), timestamp);
             win.pool_notebook_go_to(type, uu);
         }
     }
@@ -658,6 +679,14 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
     else if (op == "focus") {
         present(timestamp);
     }
+    else if (op == "open-project") {
+        const auto &filename = j.at("filename").get<std::string>();
+        app.open_pool_or_project(filename, timestamp);
+    }
+    else if (op == "update-pool") {
+        const auto filenames = j.at("filenames").get<std::vector<std::string>>();
+        app.signal_pool_items_edited().emit(filenames);
+    }
     return nullptr;
 }
 
@@ -1337,7 +1366,7 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
             if (version.get_app() < version.get_file()) {
                 project_read_only = true;
             }
-            if (version.get_app() > version.get_file()) {
+            if (version.get_file() < 2) {
                 // need to create blocks.json
                 project->create_blocks();
                 project_needs_save = true;
