@@ -11,6 +11,7 @@
 #include "prj-mgr/prj-mgr_views.hpp"
 #include "pool-mgr/view_create_pool.hpp"
 #include "common/common.hpp"
+#include <thread>
 
 namespace horizon {
 using json = nlohmann::json;
@@ -38,8 +39,13 @@ public:
 
     enum class SpawnFlags { NONE = 0, READ_ONLY = (1 << 0), TEMP = (1 << 1) };
 
-    PoolProjectManagerProcess *spawn(PoolProjectManagerProcess::Type type, const std::vector<std::string> &args,
-                                     SpawnFlags flags = SpawnFlags::NONE);
+    struct SpawnResult {
+        PoolProjectManagerProcess *proc;
+        bool spawned; // if false, an existing window got raised
+    };
+
+    SpawnResult spawn(PoolProjectManagerProcess::Type type, const std::vector<std::string> &args,
+                      SpawnFlags flags = SpawnFlags::NONE);
 
     std::map<UUID, PoolProjectManagerProcess *> get_processes();
 
@@ -77,7 +83,8 @@ public:
 
     UUID get_pool_uuid() const;
     void pool_notebook_go_to(ObjectType type, const UUID &uu);
-    void open_pool(const std::string &pool_json, ObjectType type = ObjectType::INVALID, const UUID &uu = UUID());
+    void pool_notebook_show_settings_tab();
+    void open_pool(const std::string &pool_json);
     void update_pool_cache_status_now();
     const std::string &get_project_title() const;
 
@@ -184,8 +191,12 @@ private:
     std::string sock_mgr_ep;
 
     bool check_pools();
-    void check_schema_update(const std::string &base_path);
+    bool check_schema_update(const std::string &base_path);
+
     void check_pool_update(const std::string &base_path);
+    void check_mtimes(const std::string &base_path);
+    std::thread check_mtimes_thread;
+    Glib::Dispatcher check_mtimes_dispatcher;
 
     bool check_autosave(PoolProjectManagerProcess::Type type, const std::vector<std::string> &filenames);
 
