@@ -8,6 +8,7 @@
 #include "preferences_window_stock_info.hpp"
 #include "preferences_window_misc.hpp"
 #include "preferences_window_spacenav.hpp"
+#include "preferences_window_input_devices.hpp"
 #include "canvas/color_palette.hpp"
 #include "board/board_layers.hpp"
 #include "pool/pool_manager.hpp"
@@ -69,6 +70,11 @@ PreferencesWindow::PreferencesWindow(Preferences &prefs) : Gtk::Window(), prefer
         ed->show();
         ed->unreference();
     }
+    {
+        auto ed = new InputDevicesPreferencesEditor(preferences);
+        stack->add(*ed, "input_devices", "Input devices");
+        ed->show();
+    }
 #ifdef HAVE_SPNAV
     {
         auto ed = Gtk::manage(new SpaceNavPreferencesEditor(preferences));
