@@ -24,8 +24,9 @@ void PoolUpdater::update_entity(const std::string &filename)
 {
     try {
         status_cb(PoolUpdateStatus::FILE, filename, "");
+        const auto rel = get_path_rel(filename);
         auto entity = Entity::new_from_file(filename, *pool);
-        const auto last_pool_uuid = handle_override(ObjectType::ENTITY, entity.uuid);
+        const auto last_pool_uuid = handle_override(ObjectType::ENTITY, entity.uuid, rel);
         if (!last_pool_uuid)
             return;
         SQLite::Query q(
@@ -41,7 +42,7 @@ void PoolUpdater::update_entity(const std::string &filename)
         q.bind("$prefix", entity.prefix);
         q.bind("$pool_uuid", pool_uuid);
         q.bind("$last_pool_uuid", *last_pool_uuid);
-        q.bind("$filename", get_path_rel(filename));
+        q.bind("$filename", rel);
         q.bind_int64("$mtime", get_mtime(filename));
         q.step();
         for (const auto &it_tag : entity.tags) {
@@ -54,6 +55,9 @@ void PoolUpdater::update_entity(const std::string &filename)
     catch (const std::exception &e) {
         status_cb(PoolUpdateStatus::FILE_ERROR, filename, e.what());
     }
+    catch (const CompletePoolUpdateRequiredException &e) {
+        throw;
+    }
     catch (...) {
         status_cb(PoolUpdateStatus::FILE_ERROR, filename, "unknown exception");
     }
