@@ -23,8 +23,9 @@ void PoolUpdater::update_symbol(const std::string &filename)
 {
     try {
         status_cb(PoolUpdateStatus::FILE, filename, "");
+        const auto rel = get_path_rel(filename);
         auto symbol = Symbol::new_from_file(filename, *pool);
-        const auto last_pool_uuid = handle_override(ObjectType::SYMBOL, symbol.uuid);
+        const auto last_pool_uuid = handle_override(ObjectType::SYMBOL, symbol.uuid, rel);
         if (!last_pool_uuid)
             return;
         SQLite::Query q(pool->db,
@@ -37,7 +38,7 @@ void PoolUpdater::update_symbol(const std::string &filename)
         q.bind("$unit", symbol.unit->uuid);
         q.bind("$pool_uuid", pool_uuid);
         q.bind("$last_pool_uuid", *last_pool_uuid);
-        q.bind("$filename", get_path_rel(filename));
+        q.bind("$filename", rel);
         q.bind_int64("$mtime", get_mtime(filename));
         q.step();
         add_dependency(ObjectType::SYMBOL, symbol.uuid, ObjectType::UNIT, symbol.unit->uuid);
@@ -45,6 +46,9 @@ void PoolUpdater::update_symbol(const std::string &filename)
     catch (const std::exception &e) {
         status_cb(PoolUpdateStatus::FILE_ERROR, filename, e.what());
     }
+    catch (const CompletePoolUpdateRequiredException &e) {
+        throw;
+    }
     catch (...) {
         status_cb(PoolUpdateStatus::FILE_ERROR, filename, "unknown exception");
     }
