@@ -17,7 +17,6 @@
 #include "util/warning.hpp"
 #include "parameter/program_polygon.hpp"
 #include "common/picture.hpp"
-#include <fstream>
 #include <map>
 #include <set>
 #include <vector>
@@ -124,6 +123,9 @@ public:
     void save_pictures(const std::string &dir) const;
     void load_pictures(const std::string &dir);
 
+    std::vector<Pad *> get_pads_sorted();
+    std::vector<const Pad *> get_pads_sorted() const;
+
 private:
     void update_refs();
 };
